C------------------------------------------------------------------    
CProgram Name: CDsstr.for (Variable Selection)                      
CWritten by Manavalan Parthasarathy, Arazdordi Toumadje, and W Curtis Johnson                    
C                  
C ------------------------------------------------------------------    
C
C   Version 1.8 (formerly VSbsCD8.for)
C
C Written for an IBM computer and to be compiled by Microsoft Fortran
C Powerstation
C
CThis version incorporates random choice into variable selection, uses a minimum
c basis set and self consistency, and chooses the combinations that give alpha
C helix similar to that predicted by Hennessey and Johnson SVD
C with self consistency.
C
c ---------------------------------------------------------------------------
c   Program modified by N Sreerama for any FORTRAN compiler (introducing
c	a generic random number generator) and reading INPUT and DATA files
c	used in SELCON3
c ---------------------------------------------------------------------------
C
CNOTSTRICT
CCommand $NOTSTRICT allows us to use meta commands GetTim, Seed, and Random (in subroutine
CRndm).  These are subroutines supported by MS Powerstation.  If you are using a different
Ccompiler, these subroutines might have a different name or calling convention, or might
Cnot be supported at all.
c Modified for a generic Random number generator
c ---------------------------------------------------------------------------
C                  
CTHIS PROGRAM COMPUTES THE SECONDARY STRUCTURE OF A PROTEIN            
CFROM ITS CD, BY USING THE METHOD OF J. P. HENNESSEY, Jr.             
CAND W. C. JOHNSON, Jr. (Biochem. (1981) 20, 1085-1094) AND THE  
CSVD ALGORITHM ( L. A. Compton and W. C. Johnson, Jr, Anal.   
CBiochemistry (1986) 155, 155-167), FOR COMBINATIONS OF THE PROTEINS IN
CA MINIMUM BASIS SET (P. Manavalan  
Cand W. C. Johnson, Jr., Anal Biochem (1987) 167, 76-85).  IT IS SELF CONSISTENT
CAS (N. Sreerama and R. W. Woody, Anal Biochem (1993) 209, 32-44), USES A
CMINIMUM BASIS SET AS IS IMPLICIT IN (I. H. M. van Stokkum, H. J. W. Spoelder,
CM. Bloemendal, R. van Grondelle, and F. C. A. Groen, Anal Biochem (1990)
C191, 110-118), AND CHOOSES THIS MINIMUM BASIS SET RANDOMLY AS SUGGESTED BY
C(B. Dalmas and W. H. Bannister, Anal Biochem (1995) 225, 39-48).                
C                  
CTHIS PROGRAM CALLS THE SUBROUTINE SVD (SINGULAR VALUE DECOMP).       
C                  
CThis version analyzes the CD of a set of up to test 100 proteins in file proCD.dta
Cusing basis proteins in file basCD.dta and their corresponding secondary stucture
Cin file secstr.dta.  It randomly chooses NbsCDf basis proteins from the NbsCDi
Cproteins in the basis set to analyze the test protein. It is self-consistent with
Cthe test protein in the SVD matrix. The first ncomb calculated sets of secondary
Cstructures with 0.952<=Tot=>1.05 and RMSE<=RmseMx are searched for alpha helix
Csimilar to the alpha helix predicted by H and J SVD.  The average of the choosen
Ccombinations are stored in file anal.out.  The average of the reconstructed CD
Cfor the choosen combinations are stored in reconCD.out. 
C                  
C    INPUT DATA VARIABLES                      
C    ---------------------                      
C                 
C BasCD:    CD spectra for the proteins in the basis set                   
C        to be used in the analysis
C FssMin:   The greatest negative value allowed for a sec structure
C Fsstr:    Fractions of sec structure for each protein in basis set
c  H:        Fraction of alpha helix from H and J SVD       
C icombf:   Program stops after icombf combinations if the number of successful
C        combinations does not reach ncomb           
C icombi:   Initial combination number for this analysis, here set = 1
C Matxu and Matxv =1 to calc U and V in subroutine SVD                     
C NbasCD:   Number of proteins in basis set                      
C NbsCDv:   Number of basis CD vectors.  ONLY CD BASIS VECTORS WITH        
C        SIGNIFICANT SINGULAR VALUES SHOULD BE USED IN AN ANALYSIS      
C NbsCD1:   Dimensioned number of CD basis proteins                      
C NbsCDf:   Final number basis proteins in set after some eliminated       
C        for this analysis
C NCD:      Number of CD spectra undergoing SVD.  With self-consistency
C        this is NbsCDf+1 (including the test protein)
C ncomb:    The number of successful combinations to be averaged
C Npro:     The number of test proteins in proCD.dta                               
C Nsstr:    Number of different secondary structures(eg. H,G,E,T,P,O =6)
C Nsstr1:   Dimensioned number of secondary structures     
C Numbas:   Numbers corresponding to the basis proteins being used in SVD                    
C Nwave:    Number of CD data points (number of wavelengths)              
C Nwave1:   Dimensioned number of CD data points                      
C Proname:  Name of each protein in input data (max is 40 characters)      
C ProCD:    CD data for protein to be analysed                      
C ProFss:   Predicted fraction of sec struc for protein analyzed; the average
C        of all the successful combinations
C Psstr:    Predicted fractions stored for each successful combination
C RecnCD:   Average of the reconstructed CD for all the successful combinations
C RecnCD1:  Reconstructed CD stored for each successful combination          
C RMSE:     Average of Root Mean Square of Error at each wavelength        
C        when comparing measured ProCD to CD reconstructed        
C        from analysis for sec struc of one combination
C RmseMx:   The maximum value of RMSE for which a calculated set of
C        secondary structures will be considered
c CalCD :   Calculated CD spectra correspondong to the structure
C
C        *** The program will accept spectral data with the wavelengths
C        in any direction, but all data must be consistent.
C        The basis CD spectra supplied with this new version have the 
C        INITIAL WAVELENGTH as the LONGEST WAVELENGTH, so your protein
C        CD must begin with the longest wavelength if you are using the
C        new basis CD spectra.
C  
C        SECONDARY STRUCTURES ARE:                      
C        H:alpha-Helix; G:3/10-Helix; E:Extended-beta-strand;T:beta-Turns;
CP:Polyproline-like 3/1-helix; O:OTHERS.  TOT is the total of all secondary
Cstructures with any negative values excluded.        
C                  
c ---------------------------------------------------------------------------
c  Dimensions:  Number of proteins = 50; No. of SS = 6
c ---------------------------------------------------------------------------
      Parameter (Nwave1=100, NbsCD1=61, Nsstr1=6, Matxu=1, Matxv=1)
c      Parameter (NbsCDv=5,Nsstr=6,combi=1)
      Parameter (NbsCDv=5,combi=1)
      Parameter (RmseMx=0.25,FssMin=-0.03,NbsCDf=8)                      
      Dimension ProCD(Nwave1),BasCD(Nwave1,NbsCD1),Fsstr(NbsCD1,Nsstr1)
      Dimension Psstr(Nsstr1,400),ProFss(Nsstr1)
      Dimension RecnCD(Nwave1),RecnCD1(Nwave1,400)
      dimension CalCD(Nwave1,400),CalCDX(Nwave1)
      Character*50 Proname                      
      Character*4 Str(6),IProt(60),Test
      Character*5 RefSet
c
c
	Call Message(1)
C                  
c ---------------------------------------------------------------------------
C Read in CD spectra and secondary structures for proteins in basis set       
C               
c     Open (Unit=1, file='proCD.dta', status='OLD')                      
c     Open (Unit=2, file='basCD.dta', status='OLD')
c     Open (Unit=3, file='secstr.dta', status='OLD')
c     Open (Unit=6, file='CDsstr.out', status='OLD')
	Nin = 3
      Open (Unit=Nin, file='INPUT')
      Open (Unit=7, file='CDsstr.out')
      Open (Unit=9, file='reconCD.out')
c
	IData = 0
1	CONTINUE
 	read(Nin,*,end=99)
 	backspace(Nin)
	Idata = IData + 1
C
c  basCD.dta: the CD of the basis proteins used in the analysis
c  secstr.dta: the corresponding secondary structures for the
c     basis proteins
c  proCD.dta: the CD of the protein to be analyzed
c  anal.out: predicted secondary structure for proCD as output
c  reconCD: the reconstructed CD for the protein analyzed as output
c
c ---------------------------------------------------------------------------
c     Write(6,2050)
c     Read(5,1000) NbasCD
 2050 Format(' The basis set of protein CD spectra should be stored in f
     2ile basCD.dta, '/' the corresponding secondary structures in file 
     3secstr.dta,'/' and your CD spectrum to be analyzed in file proCD.d
     4ta with a title line'/' and then the CD data in delta epsilon unit
     5s beginning at long wavelength'/' (260nm as supplied) and at 1nm i
     6ntervals to 178nm (as supplied) '/' with each data point on a diff
     7erent line in F10.x format.'///' Type in the number of CD spectra 
     8in the basis set,'/' (NbasCD) as supplied this is 22'/)
C
c     Write(6,2051)
c     Read(5,1000) Nwave
 2051 Format(///' Your protein CD in file proCD.dta should be in delta e
     2psilon units.'/' It should begin at 260nm with 1nm intervals and e
     3nd at 178nm.'/' Otherwise you must set up a basCD file consistent 
     4with your data'/' using the basis protein data from 260 to 178 nm
     5at 1 nm intervals'/' provided in file basCD.dta.'/' This program w
     6orks reasonably well for data truncated even to 200nm,'/' but the 
     7more data you have the better your answer will be.'///' Type in th
     8e number of wavelengths (Nwave) in basCD.dta, as supplied this is 
     983.'/)
C
c     Write(6,2056)
c     Read(5,1000) Npro
 2056 Format(///' Type in the number of protein CD spectra in proCD.dta 
     2(Npro),100 maximum'/)
C
	NPRO = 1
	NCOMB = 400
c
c     Write(6,2053) Ncomb
c     Read(5,1000) ncomb
 2053 Format(///' Original input and secondary structure output will be 
     2found in file anal.out.'/' Average reconstructed CD spectrum will
     3be found in file reconCD.out'///' Type in the number of successful
     4 combinations to be considered (ncomb),'/' usually 100, but for al
     5l alpha proteins, use 400,'/' the maximum for which the program is
     6 dimensioned.'/' The number of predicted structures averaged shoul
     7d be at least 10.'//'  Number of combinations=',I5) 
C
C
	ICOMBF = 100000
c
c     Write(6,2052) ICombf
c     Read(5,1000) icombf
 2052 Format(///' Most protein CD spectra being analyzed will reach ncom
     2b=100 very quickly,'/' but if an exceptional protein takes 100,000
     3 trials, it will take about 20 m'/' on a Pentium 200MHz computer,
     4operating in Windows.'/' Truncating basCD and proCD to 240 to 178n
     5m '/' will reduce the time to about 15 m, and increasing the inter
     6val to 4nm '/' will reduce the time to about 6 m. Reducing the dat
     7a to 15 points at 4nm '/' intervals between 240 and 178 nm does no
     8t change the analysis significantly.'/' Running the program on the
     9 DOS prompt further reduces the time by one third.'///' Type in th
     1e maximum number of trial combinations allowed',
     2/' (icombf) before the program stops trying to analyze a protein,
     3'/' usually 100,000.'//'  Maximum # of Trial Combinations=',I7)
C
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c  Initialize the parameters; read input; read data files
c
	CALL CDinit(NbasCD, Nwave, Proname, BasCD, Fsstr, ProCD, 
     1  Wbegin, IPrint, IBasis, Nsstr, Str, ISeg,Test,RefSet,IProt,Nin
     2  ,Idata)
	if (NBasCD.LE.20) then
	  Ifct8 = 40320
	  Call Fctrl(NBasCD,FctNp)
	  Call Fctrl(NBasCD-8,FctNp1)
	  MxIcmb = Int(FctNP / (IFct8 * FctNp1))
	  If (Icombf.GT.MxIcmb) Icombf = MxIcmb
	endif
c
	do is=1,Nsstr
	  ProFss(is) = 0.0
	enddo
c
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      Write(7,2054) NbasCD,Nwave,Npro,ncomb,icombf, Nsstr
c     Write(*,2054) NbasCD,Nwave,Npro,ncomb,icombf, Nsstr
c
 2054 Format(//' OUTPUT FROM CDsstr Program'//' NbasCD=',I3,3x,
     1  'Nwave=',I3,3x,'Npro=',I3,3x,'ncomb=',I3,
     23x,'icombf=',I6,' Nsstr=',i3)
c
c  Basis Proteins
	write(7,2064)(IProt(i),i=1,NBasCD)
2064	format(//'  Basis Set Proteins'/(10(2x,a4,2x)))
c
c  READING DATA FILES DONE in CDinit subroutine
c
c  OUTPUT CD spectra of proteins in basis set           
	If(Iprint.NE.0) then
	write(7,'(20h  Basis Set Proteins)')
      DO 10 I=1, NbasCD                      
c     Read(2,1006) Proname
c     Write(7,1008) Proname                     
c     Read(2,1010) (BasCD(J,I),J=1,Nwave)
	write(7,'(9h Protein:,a4)')Iprot(i) 
      Write(7,2015) (BasCD(J,I),J=1,Nwave)                      
   10 Continue
c
c  OUTPUT secondary structure of proteins in basis set                       
      DO 30 I=1,NbasCD                      
c     Read(3,1006) Proname
c     Write(7,1008) Proname         
c     Read(3,1010) (Fsstr(I,K),K=1,Nsstr)
	write(7,'(9h Protein:,I4)') I
      Write(7,2010) (Fsstr(I,K),K=1,Nsstr)                      
   30 Continue
	EndIf
C
C Loops thru up to 100 proteins to be analyzed
c     Do 1202 IProCD=1,Npro
C
C Initialize reconstucted CD And Calculated CD
      Do 35,J=1,Nwave
	CalCDX(J) = 0.0
   35   RecnCD(J)=0.0
C                  
C Read in CD of protein to be analysed                      
C                  
c     Read(1,1006) Proname
c     Write(7,2000)                      
c      Write(7,1008) Proname
c     Write(9,1008) Proname       
c     Read(1,1010) (ProCD(J),J=1,Nwave)                      
c      Write(7,2015) (ProCD(J),J=1,Nwave)
C
C Analyze ProCD using all basis CD spectra in basCD as original Hennessey and
C Johnson to get accurate analysis for alpha helix (H)
c     Call BVPRO3(NbasCD,Nwave,BasCD,Fsstr,Nsstr,ProFss,
c    2NbsCDv,ProCD)
      Call BVPRO3(NbasCD,Nwave,BasCD,Fsstr,Nsstr,ProFss,
     2 5,ProCD)
c
       H=ProFss(1)
       If(IBasis.NE.2.AND.IBasis.NE.5)
     1           H=ProFss(1)+ProFss(2)
       Write(7,2060) H
c       Write(*,2060) H

c	Write(7,*)ProFss
c 	STOP
c
 2060 Format(/80(1h_)//
     1   ' Helix from H and J SVD with all basis proteins is ',F6.3,/)
	If(IPrint.NE.0) write(7,4109)
4109 	format(
     2    80(1h_)/' Successful Combinations:'/
     4   ' IComb# ISucs#    Proteins 1-8'/)
C                 
C Set up seed for generation random numbers (CWin - Windows)
CWin	Call GetTim(ihr,imin,isec,i100th)                        !Windows
CWin      iseed=(360000*ihr)+(6000*imin)+(100*isec)+i100th       !Windows
CWin      Call Seed(iseed)                                       !Windows
C
C  
      NbsCDi=NbasCD
      icnt=0
C
      DO 1200 icomb=1,icombf
c                        
      Call BVPRO4(Nwave,BasCD,Fsstr,Nsstr,Psstr,RecnCD1,IproCD,
     2RmseMx,FssMin,NbsCDv,ProCD,NbsCDi,NbsCDf,icnt,icomb,icombf,H,
     3Iprint,CalCD)
      If(icnt.ge.ncomb) GoTo 1205
 1200 Continue
 1205      If (icomb.ge.icombf) then
           icntf=icnt
           Else
           icntf=ncomb
           Endif
             If(icntf.lt.5) then
             Write(7,2500)
             Write(7,2501)
             GoTo 1201
             Endif
C
C The program now searches the ncomb successful combinations for ones that have
C a fraction of alpha helix similar to that predicted by Hennessey and Johnson
C SVD (H).  For most proteins with H less than 0.15 it doesn't matter, but a few
C have alpha helix closer to the minimum in the successful combinations while
C others have alpha closer to H.  We average these two possibilities to get the
C basis for the search.  Proteins with alpha greater than 0.65 need all the alpha
C they can get, and we search the successful combinations for the maximum alpha.
C For most proteins with H between 0.25 and 0.65 it doesn't matter, but a few
C have alpha closer to the maximum in the successful combinations while
C others have alpha closer to H.  We average these two possibilities to get the
C basis for this search.  The search of the combinations for proteins with H
C between 0.15 and 0.25 simply uses H as the basis.
C
c
	write(7,4010)(Str(k),k=1,Nsstr)
c	write(*,4010)(Str(k),k=1,Nsstr)

4010	format(/80(1h_)//'  Valid Solutions (combinations):'/
     1   ' Number  SComb',6(2x,a4))
c     1   ' Number  SComb  Str1  Str2  Str3  Str4  Str5  Str6  '/)
c	
c Inititalize ProFss now!
	do i=1,6
	  ProFss(i) = 0.0
	enddo
c Sreeram
      min=icntf/20
          If(min.lt.4) min=4
        If(H.lt.0.15) then
        Hmin=1.0
        Do 40 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If(Hvalu.lt.Hmin)  Hmin=Hvalu
40	Continue
   43   If(Hmin.le.0.0) Hmin=0.0
        Hmin=(Hmin+H)/2.0
        Hmax=Hmin+0.03
        Hmin=Hmin-0.03
         Write(7,2017)Hmin,Hmax
        isstr=0
        Do 45 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If((Hvalu.ge.Hmin).and.(Hvalu.le.Hmax)) then
          Do 46 K=1,Nsstr
   46     ProFss(K)=Psstr(K,I)+ProFss(K)
          Do 141 J=1,Nwave
	    CalCDX(J) = CalCDX(J) + CalCD(J,I)
  141     RecnCD(J)=RecnCD(J)+RecnCD1(J,I)
          isstr=isstr+1
c	  write(7,3090)isstr,i,(Psstr(j,i),j=1,6)
 	  write(7,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)
c 	  write(*,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)

          Endif
   45   Continue
             If(isstr.eq.0) then
             Write(7,1025)
             Goto 1201
             Endif
 1025 Format(' Random selection failed to give good helix, try again')
        Do 47 K=1,Nsstr
   47   ProFss(K)=ProFss(K)/isstr
	RMSCDx = 0.0
        Do 142 J=1,Nwave
	   CalCDX(J) = CalCDX(J)/ isstr
	   RMSCDx = RMSCDx + (CalCDX(J) - ProCD(J))**2
  142   RecnCD(J)=RecnCD(J)/isstr
	RMSCDx = (RMSCDx/Float(Nwave))**0.5
c       Write(9,1010) (RecnCD(J),J=1,Nwave)
	write(9,1009)
        Write(9,1010) (Wbegin-FLOAT(J-1),ProCD(J),RecnCD(J),
     1                  CalCDX(J),J=1,Nwave)
c       Write(*,2011) (Str(k),ProFss(K),K=1,Nsstr)
c       Write(*,2111) icntf,isstr

        Write(7,2011) (Str(k),ProFss(K),K=1,Nsstr)
        Write(7,2111) icntf,isstr
cWrite(7,2122) RMSCDx
cx	write(4,1008)Proname
	sumss = 0
	do k=1,6
	  sumss = sumss + profss(k)
	enddo
c        Write(4,3091) Test,(ProFss(K),K=1,Nsstr)
C
        ElseIf(H.gt.0.25) then
        Hmax=0.0
        Do 50 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If(Hvalu.gt.Hmax) Hmax=Hvalu
50	continue
   51   icnt=0
        Do 52 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If(Hvalu.ge.Hmax) icnt=icnt+1
   52   Continue
          If(icnt.ge.min) GoTo 53
          Hmax=Hmax-0.001
          GoTo 51
   53     If(H.lt.0.65) then
            Hmax=(Hmax+H)/2.0
            Hmin=Hmax-0.03
            Hmax=Hmax+0.03
          Else
            Hmin=Hmax-0.03
            Hmax=1.0
          Endif
         Write(7,2017)Hmin,Hmax
        isstr=0
        Do 55 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If((Hvalu.ge.Hmin).and.(Hvalu.le.Hmax)) then
          Do 56 K=1,Nsstr
   56     ProFss(K)=Psstr(K,I)+ProFss(K)
          Do 151 J=1,Nwave
	    CalCDX(J) = CalCDX(J) + CalCD(J,I)
  151     RecnCD(J)=RecnCD(J)+RecnCD1(J,I)
          isstr=isstr+1
c	  write(7,3090)isstr,i,(Psstr(j,i),j=1,6)
 	  write(7,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)
c 	  write(*,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)

          Endif
   55   Continue
             If(isstr.eq.0) then
             Write(7,1025)
             Goto 1201
             Endif
        Do 57 K=1,Nsstr
   57   ProFss(K)=ProFss(K)/isstr
	RMSCDx = 0.0
        Do 152 J=1,Nwave
	   CalCDX(J) = CalCDX(J) / isstr
	   RMSCDx = RMSCDx + (CalCDX(J) - ProCD(J))**2
  152   RecnCD(J)=RecnCD(J)/isstr
	RMSCDx = (RMSCDx/Float(Nwave))**0.5
c       Write(9,1010) (RecnCD(J),J=1,Nwave)
	write(9,1009)
        Write(9,1010) (Wbegin-FLOAT(J-1),ProCD(J),RecnCD(J),
     1                  CalCDX(J),J=1,Nwave)
c        Write(7,2011) (ProFss(K),K=1,Nsstr),icntf,isstr
c        Write(*,2011) (Str(k),ProFss(K),K=1,Nsstr)
c        Write(*,2111) icntf,isstr

        Write(7,2011) (Str(k),ProFss(K),K=1,Nsstr)
        Write(7,2111) icntf,isstr
cWrite(7,2122) RMSCDx
cx	write(4,1008)Proname
	sumss = 0
	do k=1,6
	  sumss = sumss + profss(k)
	enddo
c        Write(4,3091) Test,(ProFss(K),K=1,Nsstr)
C
      Else
        Hmin=H-0.03
        Hmax=H+0.03
         Write(7,2017)Hmin,Hmax
              isstr=0
        Do 65 I=1,icntf
          If(IBasis.NE.2.AND.IBasis.NE.5) then
             Hvalu = Psstr(1,I)+Psstr(2,I)
	  Else
             Hvalu = Psstr(1,I)
	  Endif
          If((Hvalu.ge.Hmin).and.(Hvalu.le.Hmax)) then
          Do 66 K=1,Nsstr
   66     ProFss(K)=Psstr(K,I)+ProFss(K)
          Do 161 J=1,Nwave
	    CalCDX(J) = CalCDX(J) + CalCD(J,I)
  161     RecnCD(J)=RecnCD(J)+RecnCD1(J,I)
          isstr=isstr+1
c	  write(7,3090)isstr,i,(Psstr(j,i),j=1,6)
 	  write(7,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)
c 	  write(*,3090)isstr,i,(Psstr(j,i),j=1,Nsstr)

          Endif
   65   Continue
             If(isstr.eq.0) then
             Write(7,1025)
             Goto 1201
             Endif
        Do 67 K=1,Nsstr
   67   ProFss(K)=ProFss(K)/isstr
	RMSCDx = 0.0
        Do 162 J=1,Nwave
	    CalCDX(J) = CalCDX(J) / isstr
	   RMSCDx = RMSCDx + (CalCDX(J) - ProCD(J))**2
  162   RecnCD(J)=RecnCD(J)/isstr
	RMSCDx = (RMSCDx/Float(Nwave))**0.5
c       Write(9,1010) (RecnCD(J),J=1,Nwave)
	write(9,1009)
        Write(9,1010) (Wbegin-FLOAT(J-1),ProCD(J),RecnCD(J),
     1                  CalCDX(J),J=1,Nwave)
c        Write(7,2011) (ProFss(K),K=1,Nsstr),icntf,isstr
c        Write(*,2011) (Str(k),ProFss(K),K=1,Nsstr)
c        Write(*,2111) icntf,isstr

        Write(7,2011) (Str(k),ProFss(K),K=1,Nsstr)
        Write(7,2111) icntf,isstr
cWrite(7,2122) RMSCDx
cx	write(4,1008)Proname
	sumss = 0
	do k=1,6
	  sumss = sumss + profss(k)
	enddo
c        Write(4,3091) Test,(ProFss(K),K=1,Nsstr)
      EndIf
c
c Segments
c
	  If(Iseg.NE.0) then
	     CALL Segment(ProFss,NSSTR,AHel,BStr,HelLn,BetLn)
	     write(7,'(/80(1h_)/)')
	     write(7,452)AHel,BStr
	     write(7,453)
	     write(7,456)HelLn,BetLn
	  EndIf
453        format(//'  For YOUR PROTEIN multiply No. of SEGMENTS by'
     1     ,/'  (Number of Residues) / 100'/'  e.g., If Number of',
     2     ' Residues = 153,  Use a FACTOR of 1.53',//
     3     '  Average Length of Segments Remains as Estimated') 
452	 format(/5x,'+',50('-'),'+',/,5x,'|  Based on the SOLUTION',
     1     ' the Number of SEGMENTS of |',/,5x,'|   HELICES (Per 100',
     2     ' Residues) are:',f10.3,5x,' |',/5x,'|   STRANDS (Per 100',
     3     ' Residues) are:',f10.3,5x,' |',/5x,'+',50('-'),'+')
456	 format(/5x,'+',50('-'),'+',/,5x,'|  Based on the SOLUTION',
     1     ' Obtained                  |'
     2     /5x,'|   The AVERAGE LENGTH of HELICES :',f10.3,5x,' |',
     3     /5x,'|   The AVERAGE LENGTH of STRANDS :',f10.3,5x,' |',
     4     /5x,'+',50('-'),'+')
	   write(7,'(/80(1h_)/)')

C
c RMSD
	RmsCD = 0.0
	do k=1,Nwave
	  RmsCD = RmsCD + (ProCD(k) - CalCDX(k))**2
	enddo
	RmsCD = (RmsCD / Float(Nwave))**0.5
c
 	  Rms_N = rms_nrm(ProCD,CalCDX,Nwave,MxCD)
c
	CALL Plot(ProCD,CalCDX,Wbegin,WBegin-Nwave+1)
	Write(7,2122) RMSCD 
	Write(7,2123) RMS_N
        Write(7,440)
c
c        Write(4,3091) Test,(ProFss(K),K=1,Nsstr),RmsCD,Rms_N
c        Write(*,3091) Test,(ProFss(K),K=1,Nsstr),RmsCD,Rms_N
	Open(unit=10,file='ProtSS.out',access='APPEND')
c891	read(10,1008,end=892)
c	goto 891
c892	continue 
      Write(10,440)
440     format(80(1H_)/)
        Write(10,'(16H SAMPLE:       :,a50)')Proname
        Write(10,'(27H PROGRAM:      :   CDSSTR  )')
        Write(10,'(16H Ref. Prot. Set:,3x,A5)')RefSet
        Write(10,892)(Str(k),k=1,Nsstr)
        Write(10,894)(ProFss(k),k=1,Nsstr)
        Write(*,892)(Str(k),k=1,Nsstr)
        Write(*,894)(ProFss(k),k=1,Nsstr)
        Write(10,'(16h RMSD(Exp-Calc):,f8.3)') RmsCD
        Write(10,'(16h NRMSD(Exp-Cal):,f8.3)') Rms_N
892     format(' Structure     :',2x,6(2x,a4,2x))
894     format(' Fractions     :',6f8.3)
	Close(10)
C	
 1201 Continue
 1202 Continue
c
	Call Message(2)

c
	goto 1
99	continue
	Write(*,
     1   '(/5x,18h *****************,
     2     /5x,18h *   ALL DONE    *,
     3     /5x,18h *****************)')
c
 2000 Format(///)
 1000 Format(I10)
 1006 Format(A50)
 1008 Format(3x,A50)                      
 1009 format('  WaveL      Exptl     ReconCD    CalcCD')
 1010 Format(4F10.2)
 1012 Format(3x,'Protein being analyzed is ',A50)
 1014 Format(3x,'RmseF is:')             
 2010 Format(3X,10F6.3)
c 2011 Format(/' Predicted:  H=',F4.2,'  G=',F4.2,'  E=',F4.2,'  T=',
c     2F4.2,'  P=',F4.2,'  O=',F4.2,' isstr=',i3,' icntf=',i4)
 2011 Format(/' Predicted:'6(2x,a4,'=',f4.2))
 2111 Format(/'            isstr=',i3,' icntf=',i4)
 2122 Format(/' RMS difference between CALC. and EXPT. CD=',F7.3)
 2123 Format(/' Norm RMS diff. between CALC. and EXPT. CD=',F7.3/)
 2012 Format(3x,'Fsstr:',3x,8F6.3)
 2015 Format(3X,10F6.2)
2017  Format(1X,' Hmin:',F6.2,';  Hmax:',F6.2)                 
 2500 Format(' The number of acceptable solutions is less than five.')
 2501 Format(' There is a problem. Are you using delta epsilon units?')
 3090 Format(2X,2I6,6F6.2,1X,F6.2,1X,2F7.3)
 3091 Format(1x,a4,8f8.3)
C              
c     Close(Unit=1)
c     Close(Unit=2)
c     Close(Unit=3)
      Close(Unit=7)
      Close(Unit=9)                      
      END                      
C                  
C                  
C                  
      SUBROUTINE BVPRO3(NbasCD,Nwave,BasCD,Fsstr,Nsstr,ProFss,
     2NbsCDv,ProCD)            
      Parameter (Nwave1=100, NbsCD1=61, Nsstr1=6, Matxu=1, Matxv=1)          
      Dimension CD(Nwave1,NbsCD1), U(Nwave1,NbsCD1), V(Nwave1,NbsCD1)       
      Dimension S(NbsCD1), Work(NbsCD1), SS(NbsCD1,Nsstr1)                     
      Dimension BasCD(Nwave1,NbsCD1), Fsstr(NbsCD1,Nsstr1)         
      Dimension C(NbsCD1), ProFss(Nsstr1), ProCD(Nwave1)      
      LOGICAL Matu, Matv                     
C             
C CREATES INPUT DATA FOR SINGULAR VALUE DECOMPOSITION                   
C
      NbsCDf=NbasCD                  
      DO 12 ia=1,NbsCDf                      
      DO 3 J=1,Nwave                      
    3 CD(J,ia)=BasCD(J,ia)                      
      DO 4 K=1,Nsstr                      
    4 SS(ia,K)= Fsstr(ia,K)                      
   12 Continue
      NCD=NbsCDf+1
      DO 15 J=1,Nwave
   15 CD(J,NCD)=ProCD(J)
      DO 16 K=1,Nsstr
   16 SS(NCD,K)=0.17                      
c     SS(NCD,1)=0.36
c     SS(NCD,2)=0.06
c     SS(NCD,3)=0.18
c     SS(NCD,4)=0.12
c     SS(NCD,5)=0.07
c     SS(NCD,6)=0.21                      
      IM=1                      
      Matu=IM.EQ.Matxu                      
      Matv=IM.EQ.Matxv                     
C                  
C SUBROUTINE SVD IS CALLED FOR SINGULAR VALUE DECOMPOSITION             
C                  
C THE VARIABLE Nwave1 MUST BE SET AT LEAST AS LARGE AS THE MAXIMUM     
C OF ROW DIMENSION OF 'CD'.SEE SUBROUTINE SVD FOR FURTHER COMMENTS.   
C                
      Call SVD(Nwave,NCD,CD,S,Matu,U,Matv,V,ierr,Work,
     1         Nwave1,NbsCD1)               
C             
      IF(ierr .NE. 0) Write(7,1025) ierr                      
 1025 Format('  TROUBLE,  IERR=',I4)                      
C                  
C Predicted Structure (ProFss) from CD of protein (ProCD) and           
C significant number (NbsCDv) of CD basis vectors in U by simple        
C matrix multiplication from L. A. Compton and W. C. Johnson Jr.        
C (Anal Biochem (1986) 155, 155-167) given by:                 
C      ProFss = Fsstr * V * S(inverse) * U(transpose) * ProCD           
C                  
22    IF(NbsCDv.EQ.0) GO TO 400                      
C                  
      DO 35 ib=1,NCD                      
   35 C(ib)=0.0                    
      DO 60 K=1,NbsCDv
      Dotprd=0.0                      
      DO 40 J=1,Nwave  
   40 Dotprd=Dotprd+U(J,K)*ProCD(J)
          If(S(K).eq.0.0) then
          Write(7,1026)
          Goto 400
          Endif
 1026 Format( 'Trouble! A singular value equals 0.0, run protein again')                      
      Dotprd=Dotprd/S(K)
      DO 50 I=1,NCD                                
   50 C(I)=C(I)+Dotprd*V(I,K)                      
   60 Continue
      DO 75 JINT=1,20                  
      DO 75 K=1,Nsstr                      
      ProFss(K)=0.0        
      DO 70 I=1,NCD                      
   70 ProFss(K)=ProFss(K)+C(I)*SS(I,K)
      SS(NCD,K)=ProFss(K)             
   75 Continue                      
C
C                  
  400 Continue                      
C                  
C END OF PROGRAM                      
C       
9998  RETURN                      
      END
C
C
C                  
      SUBROUTINE BVPRO4(Nwave,BasCD,Fsstr,Nsstr,Psstr,RecnCD1,IproCD,
     2RmseMx,FssMin,NbsCDv,ProCD,NbsCDi,NbsCDf,icnt,icomb,icombf,H,
     3Iprint,CalCD)
      Parameter (Nwave1=100, NbsCD1=61, Nsstr1=6, Matxu=1, Matxv=1)         
      Dimension CD(Nwave1,NbsCD1), U(Nwave1,NbsCD1), V(Nwave1,NbsCD1)       
      Dimension S(NbsCD1), Work(NbsCD1),RecnCD1(Nwave1,400)                     
      Dimension BasCD(Nwave1,NbsCD1), Fsstr(NbsCD1,Nsstr1)       
      Dimension Numbas(NbsCD1), SS(NbsCD1,Nsstr1), Psstr(Nsstr1,400)           
      Dimension C(NbsCD1), ProFss(Nsstr1), ProCD(Nwave1),RecnCD(Nwave1)      
      dimension SF(Nsstr1),SMatU(NbsCD1,Nsstr1),SMatV(NbsCD1,Nsstr1)
     1 ,X(Nwave1),RecCD(Nwave1,NbsCD1),CalCD(Nwave1,400),RecCD1(Nwave1)
      LOGICAL Matu, Matv                     
C             
C CREATES INPUT DATA FOR SINGULAR VALUE DECOMPOSITION                   
C                
      IF(icnt.gt.icombf) STOP
      Call Rndm(NbsCDi,NbsCDf,Nwave,Nsstr,
     2BasCD,Fsstr,Numbas,CD,SS,Icomb)
      NCD=NbsCDf+1
      DO 15 J=1,Nwave
   15 CD(J,NCD)=ProCD(J)
      DO 16 K=1,Nsstr
   16 SS(NCD,K)=0.17                      
c     SS(NCD,1)=0.36
c     SS(NCD,2)=0.06
c     SS(NCD,3)=0.18
c     SS(NCD,4)=0.12
c     SS(NCD,5)=0.07
c     SS(NCD,6)=0.21                      
      IM=1                      
      Matu=IM.EQ.Matxu                      
      Matv=IM.EQ.Matxv                     
C                  
C SUBROUTINE SVD IS CALLED FOR SINGULAR VALUE DECOMPOSITION             
C                  
C THE VARIABLE Nwave1 MUST BE SET AT LEAST AS LARGE AS THE MAXIMUM     
C OF ROW DIMENSION OF 'CD'.SEE SUBROUTINE SVD FOR FURTHER COMMENTS.   
C                  
      Call SVD(Nwave,NCD,CD,S,Matu,U,Matv,V,ierr,Work,
     1         Nwave1,NbsCD1)                 
C             
      IF(ierr .NE. 0) Write(7,1025) ierr                      
 1025 Format('  TROUBLE,  IERR=',I4)                      
C                  
C Predicted Structure (ProFss) from CD of protein (ProCD) and           
C significant number (NbsCDv) of CD basis vectors in U by simple        
C matrix multiplication from L. A. Compton and W. C. Johnson Jr.        
C (Anal Biochem (1986) 155, 155-167) given by:                 
C      ProFss = Fsstr * V * S(inverse) * U(transpose) * ProCD           
C                  
22    IF(NbsCDv.EQ.0) GO TO 400                 
C                  
      DO 35 ib=1,NCD                      
   35 C(ib)=0.0                      
      DO 60 K=1,NbsCDv
      Dotprd=0.0                      
      DO 40 J=1,Nwave  
   40 Dotprd=Dotprd+U(J,K)*ProCD(J)
          If(S(K).eq.0.0) then
          Write(7,1026)
          Goto 400
          Endif
 1026 Format( 'Trouble! A singular value equals 0.0, run protein again')                      
      Dotprd=Dotprd/S(K)
      DO 50 I=1,NCD                                
   50 C(I)=C(I)+Dotprd*V(I,K)                      
   60 Continue
      DO 75 JINT=1,20                          
      Tot=0.0                  
      DO 75 K=1,Nsstr                      
      ProFss(K)=0.0             
      DO 70 I=1,NCD                      
   70 ProFss(K)=ProFss(K)+C(I)*SS(I,K)
      SS(NCD,K)=ProFss(K)                      
      IF(ProFss(K).LT.0.0) GO to 75                      
      Tot=Tot+ProFss(K)                      
   75 Continue               
C                  
C Reconstructed CD spectrum corresponding to predicted sec struc        
C                  
      SSE=0.0                      
      DO 90 J=1,Nwave
      RecnCD(J)=0.0
      DO 88 JA=1,Nwave                      
      DO 88 K=1,NbsCDv                      
   88 RecnCD(J)=RecnCD(J)+U(J,K)*U(JA,K)*ProCD(JA)    
C
      se=RecnCD(J)-ProCD(J)                                           
      SSE=SSE+se*se                      
   90 Continue                      
      RMSE=SQRT(SSE/FLOAT(Nwave))
C
C
      If((Tot.ge.0.952).and.(Tot.le.1.05).and.(RMSE.le.RmseMx).and.
     2(ProFss(1).ge.FssMin).and.(ProFss(2).ge.FssMin).and.
     3(ProFss(3).ge.FssMin).and.(ProFss(4).ge.FssMin).and.
     4(ProFss(5).ge.FssMin).and.(ProFss(6).ge.FssMin))then
      icnt=icnt+1
      Do 91 K=1,Nsstr
91    Psstr(K,icnt)=ProFss(k)
      Do 95 J=1,Nwave
   95 RecnCD1(J,icnt)=RecnCD(J)                      
c       if(Iprint.NE.0) Write(7,1020) icomb,icnt,(ProFss(K),K=1,Nsstr),Tot,RMSE
c      Write(7,1021)(Numbas(I),I=1,NbsCDf)
c      Write(7,1090)
C  Write(7,2010)(S(I),I=1,NbsCDv)
c 
c Fraction of Data included from ratio of 5 SVD to 9 SVD
c
	ssum = 0.0
	sum5 = 0.0
	do i=1,5
	  sum5 = sum5 + S(i)
	enddo
	do i=1,9
	  ssum = ssum + S(i)
	enddo
c       Write(7,2010)(S(I)/ssum,I=1,9)
c       Write(7,2010)sum5/ssum
C  Write(7,1091)
C  DO 92 K=1,NbsCDv
C92    Write(7,2010)(U(J,K),J=1,Nwave)
C  Write(7,1092)
C  DO 93 I=1,NCD
C93    Write(7,2010)(V(I,K),K=1,NbsCDv)
c
c Compute the Calcuated CD spectrum corresponding to the predicted SS
c CalcCD = ProFss * SMatV * SF(inv) * SMatU(Trns) * RecCD
c N. Sreerama
c
	Call SVD(NCD,Nsstr,SS,SF,Matu,SMatU,Matv,SMatV,Ierr,Work,
     1           NbsCD1,Nsstr1)
	do ip = 1,NCD
	  X(ip) = 0.0
	enddo
	Do is = 1,Nsstr
	  Prod = 0.0
	  do ic = 1,Nsstr
	     Prod = Prod + SMatV(ic,is) * ProFss(ic)
	  enddo
	  if(SF(is).NE.0.0) Prod = Prod / SF(is)
	  do ip = 1,NCD
	     X(ip) = X(ip) + SmatU(ip,is) * Prod
	  enddo
	EndDo
c RecCD
	do ip = 1,NCD
	 do iw = 1,Nwave
	    RecCD(iw,ip) = 0.0
 	  	do is = 1,NbsCDv
		   RecCD(iw,ip) = RecCD(iw,ip) + U(iw,is) * S(is)
     1				          * V(ip,is)
	        enddo
	 enddo
	enddo
	rmsRec = 0.0
	do iw = 1,nwave
	   RecCD1(iw) = RecCD(iw,NCD)
c	   rmsRec = rmsRec + (ProCD(iw)-RecCD1(iw))**2
c           RecnCD1(iw,icnt)=RecCD1(iw)                      
	enddo
c	rmsRec = (RmsRec / Float(Nwave))**0.5
c	write(7,*) icnt,RmsRec, RMSE
c CalCD
	do iw = 1, Nwave
	   CalCD(iw,icnt) = 0.0
	   do ip = 1,NCD
	      CalCD(Iw,icnt) = CalCD(iw,icnt) + X(ip) * RecCD(iw,ip)
	   enddo
	enddo
	

c
c  Order NumBas 
cdo j = 1,NbsCDf
cNTmpBs = Numbas(j)
cIndBs = j
cdo i=j+1,NbsCDf
c   if(NumBas(i).LT.NTmpBs) then
c	NTmpBs = NumBas(i)
c	IndBs = i
c   endif
cenddo
cNumBas(IndBs) = NumBas(J)
cNumBas(j) = NTmpBs
cenddo
c
c       Write(10,1000) icnt,(Numbas(I),I=1,NbsCDf)
c      If(IPrint.NE.0) Write( 7,1000) icomb, icnt,(Numbas(I),I=1,NbsCDf)
       If(IPrint.NE.0)Write( 7,1209) icomb, icnt,(Numbas(I),I=1,NbsCDf)
       If(IPrint.NE.0)Write(7,1020) icomb,icnt,(ProFss(K),K=1,Nsstr),
     1Tot,RMSE         
c       Write(10,1020) icomb,icnt,(ProFss(K),K=1,Nsstr),Tot,RMSE         
 1000 Format(i7,20I3)
 1209 Format(i7,i6,20I4)
      EndIf
C
      icombr=icombf-icomb
c      Write(7,3001) IproCD,icnt,icomb,icombr                    
C                  
  400 Continue                      
C                  
C END OF PROGRAM                      
C                  
1011  Format(F10.2)
1020  Format(i7,I6,6F7.3,1X,F7.3,1X,2F7.3)
c1020  Format(2X,2I6,6F6.2,1X,F6.2,1X,2F7.3)
1021  Format(3x,25I3)
1090  Format('Singular Values are:')
1091  Format('U Matrix is:')
1092  Format('V Matrix is:')
2010  Format(1X,10F6.2)                 
3001  Format(' Pro',I3,',   Successful comb',I4,',   Trial comb',I7,
     2',   Remaining comb=',I7)                      
9998  RETURN                      
      END
C
C
C
      Subroutine Rndm(NbsCDi,NbsCDf,Nwave,Nsstr,
     2BasCD,Fsstr,Numbas,CD,SS,Icount)
      Parameter (Nwave1=100, NbsCD1=61, Nsstr1=6)
      Dimension Numbas(NbsCD1),BasCD(Nwave1,NbsCD1),Fsstr(NbsCD1,Nsstr1)
     2,CD(Nwave1,NbsCD1),SS(NbsCD1,Nsstr1)
	Dimension Nseed(10),NPrCom(100000,8)
  	data init/0/
  	data Nseed/98263, 37829, 28739, 18273, 78393, 99283, 65988,
     1             32467, 44683, 53892/
c
c Find a new combinatio
c
1	continue
c
  	if (init.EQ.0) IndRnd = 1
  	If (IndRnd.EQ.11) IndRnd = 1
c
      DO 100 I=1,NbsCDf
   10 I1=0
CWin      Call Random(ranval)
c Get a random number and set the seed for the next
c If the SEED becomes <100, New SEED is Generated with MOD(Seed, 3**(8 to 0))
c  Random Number Function for SGI
  	if(iseed.EQ.0) then
c	    write(6,929)
c929	    format(' INPUT Seed for Random number, 5 digits')
c	    read(5,*) iseed1
  	    Iseed1 = Nseed(IndRnd)
  	    IndRnd = IndRnd + 1
  	    iseed = mod(iseed1,2**15)
  	endif

  	ran3vl = ran3(iseed)
  	iseed = nint(ran3vl * 2**14)
  	ipwr = 5
  	if(ihr.EQ.0) ihr = 1
  	if(iseed.EQ.0) then
  	   iseed = (ihr /  24) * 360 + Ihr * 360
  	endif
c932	if(iseed.LT.10.AND.ipwr.LT.9) then
c	   iseed = nint(ran3vl * 3**ipwr)
c	   ipwr = ipwr + 1
c	   goto 932
c	endif
  	ranval = ran(iseed)
  	iseed = nint(ranval * 2**14)
  	ipwr = 5
c933	if(iseed.LT.10.AND.Ipwr.LT.9) then
c	   iseed = nint(ranval * 3**ipwr)
c	   ipwr = ipwr + 1
c	   goto 933
c	endif
  	if (iseed.EQ.0) write(7,934)
934     format(' SEED for Random Number Has Become ZERO. New Seed.')
cseed	write(11,*)iseed,ranval
c  Sreerama, Jul 15, 98
      scrap=ranval*Real(NbsCDi)
      Numbas(I)=Int(scrap)+1
   20 I1=I1+1
      IF(I1.ge.I) Goto 30
      IF(Numbas(I).eq.Numbas(I1))Goto 10      
      Goto 20
   30 na=Numbas(I)
      DO 3 J=1,Nwave
    3 CD(J,I)=BasCD(J,na)
      DO 4 K=1,Nsstr
    4 SS(I,K)=Fsstr(na,K)
  100 Continue
C
C  Write(8,1000) icnt,(Numbas(I),I=1,NbsCDf)
 1000 Format('icnt=',I4,2x,30I3)
c  Order NumBas 
	do j = 1,NbsCDf
	NTmpBs = Numbas(j)
	IndBs = j
	do i=j+1,NbsCDf
	   if(NumBas(i).LT.NTmpBs) then
		NTmpBs = NumBas(i)
		IndBs = i
	   endif
	enddo
	NumBas(IndBs) = NumBas(J)
	NumBas(j) = NTmpBs
	enddo
c
c Find if the basis combination has been done 
c
	If (Icount.LE.1) GoTo 99
	do i=1,Icount-1
	 Irep = 0
	do j=1,NbsCDf
 	  If(Numbas(j).NE.NPrCom(i,j))then
		 Irep = 1
		 goto 98
	  endif
	enddo
98	continue
	if(irep.EQ.0) goto 1
	enddo
c	write(6,'(i7,10i4)')icount,(NPrCom(icount,j),j=1,NBsCDf)
c
99	continue
c
c copy Basis combination to NPrCom
c
	do j=1,NbsCdf
		NPrCom(icount,j) = Numbas(j)
	enddo
c	write(6,*)
c 	write(6,'(i7,10i4)')icount,(NPrCom(icount,j),j=1,NBsCDf)
c
      Return
      End
C
C
C
            SUBROUTINE SVD(M,N,A,W,Matu,U,Matv,V,ierr,RV1,
     1                     Nwave1,NbsCD1)
CNOTSTRICT        
C-----------------------------------------------------------------------
C ***NOTE WELL*** Nwave1 must be greater than or equal to NbsCD1,
C or you WILL GET THE WRONG ANSWER!!!!    
c  Dimensions Transferred
c     PARAMETER (Nwave1=100,NbsCD1=61)                   
      Dimension A(Nwave1,NbsCD1),W(NbsCD1),U(Nwave1,NbsCD1)                 
      Dimension V(Nwave1,NbsCD1),RV1(NbsCD1)                 
      LOGICAL MATU,MATV                   
C------------------------------------------------------------------------   
C  Singular Value Decomposition subroutine from:                   
C  Forsythe, Malcolm, & Moler. "Computer Methods for Mathematical       
C  Computations", Prentice-Hall, Inc., 1977.                   
C               
C  This subroutine is a translation of the ALGOL procedure SVD,         
C  Num. Math. 14, 403-420(1970) by Golub and Reinsch.                   
C  Handbook for Auto. Comp., Vol. II-Linear Algebra, 134-151(1971).     
C               
C  This Subroutine determines the Singular Value Decomposition         
C           T                   
C  A = U S V of a real M by N rectangular matrix.  Householder          
C  bidiagonalization and a variant of the QR algorithm are used.        
C               
C  For input:                   
C               
C        NM must be set to the row dimension of two-dimensional         
C          array parameters as declared in the calling program          
C          dimension statement.  Note that NM must be at least          
C          as large as the maximum of M and N.                   
C               
C        M is the number of rows of A, and U.                   
C               
C        N is the number of columns of A and U and the order of V.      
C               
C        A contains the rectangular input matrix to be decomposed.      
C               
C        MATU should be set to .TRUE. if the U matrix is desired.      
C               
C        MATV should be set to .TRUE. if the V matrix is desired.       
C               
C  On output:                   
C               
C        A is unaltered (unless overwritten by U or V).                
C               
C        W contains the N non-negative singular values of A (the        
C          diagonal elements of S). They are unordered.  If an error    
C          exit is made, the singular values should be correct for      
C          indicies IERR+1, IERR+2, ..., N.                   
C               
C        U contains the matrix U (orthogonal column vectors) of the     
C          decomposition if MATU has been set to .TRUE. otherwise       
C          U is used as a temporary array.  U may coincide with A.      
C          If an error exit is made, the columns of U corresponding     
C          to indices of correct singular values should be correct.    
C               
C        V contains the matrix V (orthogonal) of the decomposition if  
C          MATV has been set to .TRUE.  otherwise V is not referenced.  
C          V may also coincide with A if U is not needed.  If an error  
C          exit is made, the columns of V corresponding to indices of   
C          correct singular values should be correct.                   
C               
C        IERR is set to:                   
C              0 for normal return.                   
C              K if the K-th singular value has not been determined     
C                after 65 iterations.                   
C               
C        RV1 is a temporary storage array.                   
C               
C  Questions and comments should be directed to B. S. Gargow,        
C  Applied Mathematics Division, Argonne National Laboratory.           

C----------------------------------------------------------------------------

      IERR = 0                  
C               
      DO 100 I=1,M                   
            DO 100 J=1,N                   
                  U(I,J) = A(I,J)                   
100      CONTINUE                   
C-------------------------------------------------                   
C  HOUSEHOLDER REDUCTION TO BIDIAGONAL FORM                   
C-------------------------------------------------                   
      G = 0.0                   
      SCALE = 0.0                   
      ANORM = 0.0                   
C               
      DO 300 I=1,N                   
            L = I + 1                   
            RV1(I) = SCALE * G                   
            G = 0.0                   
            S = 0.0                   
            SCALE = 0.0                   
            IF(I .GT. M) GOTO 210                   
C               
            DO 120 K=I,M                   
120                  SCALE = SCALE + ABS( U(K,I) )                   
C               
            IF(SCALE .EQ. 0.0) GOTO 210                   
C               
            DO 130 K = I,M                   
                  U(K,I) = U(K,I) / SCALE                   
                  S = S + U(K,I)**2                   
130            CONTINUE                   
C               
            F = U(I,I)                   
            G = -SIGN( SQRT(S), F )                   
            H = F * G - S                   
            U(I,I) = F - G                   
            IF(I .EQ. N) GOTO 190                   
C               
            DO 150 J=L,N                   
                  S = 0.0                   
C               
                  DO 140 K=I,M                   
140                        S = S + U(K,I) * U(K,J)                   
C               
                  F = S / H                   
C               
                  DO 150 K=I,M                   
                        U(K,J) = U(K,J) + F * U(K,I)                   
150            CONTINUE                   
C               
190            DO 200 K=I,M                   
200                  U(K,I) = SCALE * U(K,I)                   
C               
210            W(I) = SCALE * G                   
            G = 0.0                   
            S = 0.0                   
           SCALE = 0.0                   
            IF(I .GT. M .OR. I .EQ. N) GOTO 290                   
C               
            DO 220 K=L,N                   
220                  SCALE = SCALE + ABS( U(I,K) )                   
C               
            IF(SCALE .EQ. 0.0) GOTO 290                   
C               
            DO 230 K = L,N                   
                  U(I,K) = U(I,K) / SCALE                   
                  S = S + U(I,K)**2                   
230            CONTINUE                   
C               
            F = U(I,L)                   
            G = -SIGN( SQRT(S), F )                   
            H = F * G - S                   
            U(I,L) = F - G                   
C               
            DO 240 K=L,N                   
240                  RV1(K) = U(I,K) / H                   
C               
            IF(I .EQ. M) GOTO 270                   
C               
            DO 260 J=L,M                   
                  S = 0.0                   
C               
                  DO 250 K=L,N                   
250                        S = S + U(J,K) * U(I,K)                   
C               
                  DO 260 K=L,N                   
                        U(J,K) = U(J,K) + S * RV1(K)                   
260            CONTINUE                   
C               
270            DO 280 K=L,N                   
280                  U(I,K) = SCALE * U(I,K)                   
C               
290            ANORM = AMAX1(ANORM,ABS(W(I))+ABS(RV1(I)))                   
300      CONTINUE                   
C---------------------------------------------------                   
C  ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS                   
C---------------------------------------------------                   
      IF(.NOT. MATV) GOTO 410                   
C---------------------------------------                   
C  FOR I = N STEP -1 UNTIL 1 DO --                   
C---------------------------------------                   
      DO 400 II=1,N                   
            I = N + 1 - II                   
            IF(I .EQ. N) GOTO 390                   
            IF(G .EQ. 0.0) GOTO 360                   
C               
            DO 320 J=L,N                   
320                  V(J,I) = (U(I,J) / U(I,L)) / G                   
C               
            DO 350 J=L,N                   
                  S = 0.0                   
C               
                  DO 340 K=L,N                   
340                        S = S + U(I,K) * V(K,J)                   
C               
                  DO 350 K=L,N                   
                        V(K,J) = V(K,J) + S * V(K,I)                   
350            CONTINUE                   
C               
360            DO 380 J=L,N                   
                  V(I,J) = 0.0                   
                  V(J,I) = 0.0                   
380            CONTINUE                   
C               
390            V(I,I) = 1.0                   
            G = RV1(I)                   
            L = I                   
400      CONTINUE                   
C--------------------------------------------------                   
C  ACCUMULATION OF LEFT-HAND TRANSFORMATIONS                   
C--------------------------------------------------                   
410      IF(.NOT. MATU) GOTO 510                   
C               
      MN = N                   
      IF(M .LT. N) MN = M                   
C               
      DO 500 II= 1,MN                   
            I = MN + 1 - II                   
            L = I + 1                   
            G = W(I)                   
            IF(I .EQ. N) GOTO 430                   
C               
            DO 420 J=L,N                   
420                  U(I,J) = 0.0                   
C               
430            IF(G .EQ. 0.0) GOTO 475                   
            IF(I .EQ. MN) GOTO 460                   
C               
            DO 450 J=L,N                   
                  S = 0.0                   
C               
                  DO 440 K=L,M                   
440                        S = S + U(K,I) * U(K,J)                   
C               
                  F = (S / U(I,I)) / G                   
C               
                  DO 450 K=I,M                   
                        U(K,J) = U(K,J) + F * U(K,I)                   
450            CONTINUE                   
C               
460            DO 470 J=I,M                   
470                  U(J,I) = U(J,I) / G                   
C               
            GOTO 490                   
C               
475            DO 480 J=I,M                   
480                  U(J,I) = 0.0                   
C               
490            U(I,I) = U(I,I) + 1.0                   
500      CONTINUE                   
C-----------------------------------------------                   
C  DIAGONALIZATION OF THE BIDIAGONAL FORM                   
C-----------------------------------------------                   
510      DO 700 KK = 1,N                   
            K1 = N - KK                   
            K = K1 + 1                   
            ITS = 0                   
C---------------------------                   
C  TEST FOR SPLITTING                   
C---------------------------                   
520            DO 530 LL=1,K                   
                  L1 = K - LL                   
                  L = L1 + 1                   
                  IF(ABS(RV1(L))+ANORM .EQ. ANORM) GOTO 565                 

           
                  IF(ABS(W(L1))+ANORM .EQ. ANORM) GOTO 540                  

530            CONTINUE                   
C-------------------------------------------                   
C  CANCELLATION OF RV1(L) IF L .GT. 1                   
C-------------------------------------------                   
540            C= 0.0                   
            S = 1.0                   
C               
            DO 560 I=L,K                   
                  F = S * RV1(I)                   
                  RV1(I) = C* RV1(I)                   
                  IF(ABS(F)+ANORM .EQ. ANORM) GOTO 565                   
                  G = W(I)                   
                  H = SQRT( F*F + G*G )                   
                  W(I) = H                   
                  C= G / H                   
                  S = -F / H                   
                  IF(.NOT. MATU) GOTO 560                   
C               
                  DO 550 J=1,M                   
                        Y = U(J,L1)                   
                        Z = U(J,I)                   
                        U(J,L1) = Y * C+ Z * S                   
                        U(J,I) = -Y * S + Z * C               
550                  CONTINUE                   
C               
560            CONTINUE                   
C----------------------------                   
C  TEST FOR CONVERGENCE                   
C----------------------------                   
565            Z = W(K)                   
            IF(L .EQ. K) GOTO 650                   
C---------------------------------------                   
C  SHIFT FROM BOTTOM 2 BY 2 MINOR                   
C---------------------------------------                   
c           IF(ITS .EQ. 35) GOTO 1000      ! Number of iterations changed to 65
            IF(ITS .EQ. 65) GOTO 1000                   
            ITS = ITS + 1                   
            X = W(L)                   
            Y = W(K1)                   
            G = RV1(K1)                   
            H = RV1(K)                   
            F = ((Y-Z)*(Y+Z)+(G-H)*(G+H)) / (2.0 * H * Y)                   
            G = SQRT( F*F + 1.0 )                   
            F = ((X-Z)*(X+Z)+H*(Y/(F+SIGN(G,F))-H))/X                   
C-------------------------------                   
C  NEXT QR TRANSFORMATION                   
C-------------------------------                   
            C= 1.0                   
            S = 1.0                   
C               
            DO 600 I1=L,K1                   
                  I = I1 + 1                   
                  G = RV1(I)                   
                  Y = W(I)                   
                  H = S * G                   
                  G = C* G                   
                  Z = SQRT(F*F + H*H)                   
                  RV1(I1) = Z                   
                  C= F / Z                   
                  S = H / Z                   
                  F = X * C+ G * S                   
                  G = -X * S + G * C               
                  H = Y * S                   
                  Y = Y * C               
                  IF(.NOT. MATV) GOTO 575                   
C               
                  DO 570 J=1,N                   
                        X = V(J,I1)                   
                        Z = V(J,I)                   
                        V(J,I1) = X * C+ Z * S                   
                        V(J,I) = -X * S + Z * C               
570                  CONTINUE                   
C               
575                  Z = SQRT(F*F+H*H)                   
                  W(I1) = Z                   
C-----------------------------------------------                   
C  ROTATION CAN BE ARBITRARY IF Z IS ZERO                   
C-----------------------------------------------                   
                  IF(Z .EQ. 0.0) GOTO 580                   
                  C= F / Z                   
                  S = H / Z                   
580                  F = C* G + S * Y                   
                  X = -S * G + C* Y                   
                  IF(.NOT. MATU) GOTO 600                   
C               
                  DO 590 J=1,M                   
                        Y = U(J,I1)                   
                        Z = U(J,I)                   
                        U(J,I1) = Y * C+ Z * S                   
                        U(J,I) = -Y * S + Z * C               
590                  CONTINUE                   
C               
600            CONTINUE                   
C               
            RV1(L) = 0.0                   
            RV1(K) = F                   
            W(K) = X                   
            GOTO 520                   
C--------------------                   
C  CONVERGENCE                   
C--------------------                   
650            IF(Z .GE. 0.0) GOTO 700                   
C               
            W(K) = -Z                   
            IF(.NOT. MATV) GOTO 700                   
C               
            DO 690 J=1,N                   
690                  V(J,K) = -V(J,K)                   
C               
700      CONTINUE                   
C               
      GOTO 1001                   
C-------------------------------------------------------                   
C  SET ERROR - NO CONVERGENCE AFTER 30 ITERATIONS                   
C-------------------------------------------------------                   
1000      IERR = K                   
1001      RETURN                   
      END


      FUNCTION RAN3(IDUM)
C     IMPLICIT REAL*4(M)
C     PARAMETER (MBIG=4000000.,MSEED=1618033.,MZ=0.,FAC=2.5E-7)
      PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1.E-9)
      DIMENSION MA(55)
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        MA(55)=MJ
        MK=1
        DO 11 I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
11      CONTINUE
        DO 13 K=1,4
          DO 12 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
12        CONTINUE
13      CONTINUE
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
      INEXT=INEXT+1
      IF(INEXT.EQ.56)INEXT=1
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56)INEXTP=1
      MJ=MA(INEXT)-MA(INEXTP)
      IF(MJ.LT.MZ)MJ=MJ+MBIG
      MA(INEXT)=MJ
      RAN3=MJ*FAC
      RETURN
      END
c-------------------------------------------------------------------------
c Driver subroutine for running CONTIN. Initializes the wavelength etc. 
c
	Subroutine CDinit(NbasCD, Nwave, Proname, BasCD, Fsstr, ProCD,
     1     Wbegin,IPrint,IBasis,Nclass, Str,Iseg,Test,RefSet,IProt,Nin
     2     ,Idata)
      Parameter (Nwave1=100, NbsCD1=61, Nsstr1=6, Matxu=1, Matxv=1)
      Dimension ProCD(Nwave1),BasCD(Nwave1,NbsCD1),
     1Fsstr(NbsCD1,Nsstr1)
        Character*5 RefSet
	character*50 Proname
	character*4 Str(6),Test,IProt(60)
	dimension cdd(100),sstr(6)
c Input
c
        IBasis = 0
        read(Nin,*,End=999)
        read(Nin,*)
        read(Nin,*)IPrint,IBas1
        write(7,450)
        write(7,400)
	write(*,450)
        write(*,'(/4x,9hCD DATA: ,i3)')IData
	ISkip = 0
c
c Print OPTIONS
c
c
        write(7,982)
        write(7,983)
982     format(/6x,' BASIS SET OPTIONS AVAILABLE',
     1  /6x,'_________________________________________________________'
     1  /6x,'|                                                       |'
     1  /6x,'| IBasis Ref.Set  Proteins   Secondary Structre WaveLen |'
     1  /6x,'|    1   SP29   29-SOLUBLE     H1,H2,S1,S2,T,U  178-260 |'
     2  /6x,'|    2   SP22X  22-SOLUBLE     H,3-10,S,T,P2,U  178-260 |'
     3  /6x,'|    3   SP37   37-SOLUBLE     H1,H2,S1,S2,T,U  185-240 |'
     4  /6x,'|    4   SP43   43-SOLUBLE     H1,H2,S1,S2,T,U  190-240 |'
     5  /6x,'|    5   SP37A  37-SOLUBLE       H,S,P2,T,U     185-240 |')
983      format
     1  (6x,'|    6   SDP42  SP37,5 Denatrd  H1,H2,S1,S2,T,U 185-240 |'
     7  /6x,'|    7   SDP48  SP43,5 Denatrd  H1,H2,S1,S2,T,U 190-240 |'
     8  /6x,'|    8   CLSTR  Soluble/Denatr. H1,H2,S1,S2,T,U 190-240 |'
     9  /6x,'|    9   SMP50  SP37,13 Membran H1,H2,S1,S2,T,U 185-240 |'
     1  /6x,'|   10   SMP56  SP43,13 Membran H1,H2,S1,S2,T,U 190-240 |'
     1  /6x,'|_______________________________________________________|')
c
        write(7,901)IPrint,IBas1
901     format(3x,' IPrint  IBasis  ',/2x,7i8)

c
        If(Iprint.EQ.1)Print='PRNT'
	NClass = 6
	NCD1 = 83
c
	Str(1) = 'H(r)'
	Str(2) = 'H(d)'
	Str(3) = 'S(r)'
	Str(4) = 'S(d)'
	Str(5) = 'Trn '
	Str(6) = 'Unrd'
c
c
c Intialize Basis set
c
c 29 Prot; 6 SS; Segment (178-260)
        If(IBas1.EQ.1) then
            IBasis = 1
            Iseg = 1
            WvStrt = 260.0
            WvEnd  = 178.0
            RefSet = 'SP29 '
        Endif
c 23 Prot; 6 SS;                                (178-260)
        If(IBas2.EQ.1.OR.IBas1.EQ.2) then
            IBasis = 2
            Iseg = 0
            WvStrt = 260.0
            WvEnd  = 178.0
	   Str(1) = 'AHel'
	   Str(2) = '3/10'
	   Str(3) = 'BSht'
	   Str(4) = 'Turn'
	   Str(5) = 'PP2 '
	   Str(6) = 'Unrd'
            RefSet = 'SP22X'
        Endif
c 37 Prot; 6 SS; Segment                        (185-240)
        If(IBas3.EQ.1.OR.IBas1.EQ.3) then
            IBasis = 3
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 185.0
            RefSet = 'SP37 '
        Endif
c 43 Prot; 6 SS; (From Johnson,Yang, PK set)     (190-240)
        If(IBas3.EQ.2.OR.IBas1.EQ.4) then
            IBasis = 4
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 190.0
            RefSet = 'SP43 '
        Endif
c 37 Prot; 5 SS; (For P2)                       (185-240)
        If(IBas2.EQ.2.OR.IBas1.EQ.5) then
            IBasis = 5
            Iseg = 0
            WvStrt = 240.0
            WvEnd  = 185.0
            NClass = 5
	   Str(1) = 'AHel'
	   Str(2) = 'BSht'
	   Str(3) = 'Turn'
	   Str(4) = 'PP2 '
	   Str(5) = 'Unrd'
            RefSet = 'SP37A'
        Endif
c 42 Prot; 6 SS; Segment                        (185-240)
        If(IBas3.EQ.3.OR.IBas1.EQ.6) then
            IBasis = 6
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 185.0
            RefSet = 'SDP42'
        Endif
c 48 Prot; 6 SS; Segment                        (190-240)
        If(IBas3.EQ.4.OR.IBas1.EQ.7) then
            IBasis = 7
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 190.0
            RefSet = 'SDP48'
        Endif
c CA Prot; 6 SS; Segment                        (190-240)
        If(IBas1.EQ.8) then
            IBasis = 8
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 190.0
            RefSet = 'CLSTR'
        Endif
c 50 Prot; 6 SS; Segment                        (185-240)
        If(IBas1.EQ.9) then
            IBasis = 9
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 185.0
            RefSet = 'SMP50'
        Endif
c 56 Prot; 6 SS; Segment                        (190-240)
        If(IBas1.EQ.10) then
            IBasis = 10
            Iseg = 1
            WvStrt = 240.0
            WvEnd  = 190.0
            RefSet = 'SMP56'
        Endif
c Default BASIS = 1
        If(IBasis.EQ.0.OR.IBasis.GT.10) then
            IBasis = 1
            Iseg = 1
            WvStrt = 260.0
            WvEnd  = 178.0
        Endif
c Assign Files
        If(IBasis.EQ.1) then
          write(7,781)RefSet
          write(*,781)RefSet
781       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'29 Soluble Proteins',
     2          /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.29')
          Open(unit=1,file='SSDATA.29')
	  NBasCD = 29
        EndIf
        If(IBasis.EQ.2) then
          write(7,782)RefSet
          write(*,782)RefSet
782       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'22 Soluble Proteins; Sec.Str. by XTLSSTR ',
     2          /'    Structures: A-Helix, 310-Helix, B-Sheet,',
     3          ' Turns, PP2 and Unordered'/)
          Open(unit=2,file='CDDATA.23')
          Open(unit=1,file='SSDATA.23')
	  NBasCD = 23
        EndIf
        If(IBasis.EQ.3) then
          write(7,783)RefSet
          write(*,783)RefSet
783       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'37 Soluble Proteins',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.37')
          Open(unit=1,file='SSDATA.37')
	  NBasCD = 37
        EndIf
        If(IBasis.EQ.4) then
          write(7,786)RefSet
          write(*,786)RefSet
786       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'43 Soluble Proteins',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.43')
          Open(unit=1,file='SSDATA.43')
	  NBasCD = 43
        EndIf
        If(IBasis.EQ.5) then
          write(7,787)RefSet
          write(*,787)RefSet
787       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'37 Soluble Proteins; For P2 structure Determinatn',
     2    /'    Structures: Helix, Sheet, Turns, P2, and Unordered'/)
          Open(unit=2,file='CDDATA.P2')
          Open(unit=1,file='SSDATA.P2')
	  NBasCD = 37
        EndIf
        If(IBasis.EQ.6) then
          write(7,788)RefSet
          write(*,788)RefSet
788       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'42 Proteins - 37 Soluble + 5 Denatured',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.42')
          Open(unit=1,file='SSDATA.42')
	  NBasCD = 42
        EndIf
        If(IBasis.EQ.7) then
          write(7,785)RefSet
          write(*,785)RefSet
785       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'48 Proteins - 43 Soluble + 5 Denatured',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.48')
          Open(unit=1,file='SSDATA.48')
	  NBasCD = 48
        EndIf
        If(IBasis.EQ.8) then
          write(7,789)RefSet
          write(*,789)RefSet
789       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,' Proteins Selcted from Tertiary Class',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.CA')
          Open(unit=1,file='SSDATA.CA')
	  NBasCD = 00
        EndIf
        If(IBasis.EQ.9) then
          write(7,791)RefSet
          write(*,791)RefSet
791       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'50 Proteins - 37 Soluble + 13 Membrane',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.50')
          Open(unit=1,file='SSDATA.50')
	  NBasCD = 50
        EndIf
        If(IBasis.EQ.10) then
          write(7,792)RefSet
          write(*,792)RefSet
792       format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'56 Proteins - 43 Soluble + 13 Membrane',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/)
          Open(unit=2,file='CDDATA.56')
          Open(unit=1,file='SSDATA.56')
	  NBasCD = 56
        EndIf
c
	If(ISeg.GE.1)
     1   write(7,'(/33h    NUMBER of Segments Calculated /)')
	write(7,784)
	write(*,784)
784     format(  '    SUMMARY in ProtSS.out'/
     2        '        Contains Sec. Str. Fractions, RMSD and NRMSD'
     3       /'        RMSD and Normalized-RMSD between Exp and Calc'
     4        ,' CD spectra'
     5      /'    NRMSD from the three programs generally vary in'
     6   ,' the order:'/'        CONTINLL < CDSSTR < SELCON3',
     7   /'    Use CAUTION in drawing CONCLUSIONS from RMSD/NRMSD')
c
c
	write(7,450)
	read(Nin,*)
        read(Nin,*)
        read(Nin,100) Proname
	backspace (Nin)
	read(Nin,'(a4)')Test
	write(7,100) 
	write(7,100) Proname
	read(Nin,*)
	read(Nin,*)
	read(Nin,*)Wbegin,Wend, FACTOR
	NCD = INT(wbegin-wend) + 1
	NCDN = NCD
c
	ISkip=0
	if((wbegin.GT.WvStrt).OR.(wend.LT.WvEnd))   then
		write(7,470) wbegin,wend,WvStrt, WvEnd, ncd
	   	Iskip = Wbegin - WvStrt
		if(Wbegin.GT.WvStrt) Wbegin = WvStrt
		if(Wend.LT.WvEnd) Wend = WvEnd
	      	NCDN = Int(Wbegin - Wend) + 1
	endif
	Write(*,'(/a50)')ProName
        write(*,501)wbegin,wend,ncdn
c
	read(Nin,*)
	read(Nin,*)
	read(Nin,*)(cdd(i),i=1,ncd)
	read(Nin,*)
	read(Nin,*)
	read(Nin,*)
c
	if(factor.EQ.0.0) Factor = 1.0
	 do 51 i=1,Ncdn
		ProCD(i)=cdd(i+Iskip)*factor
51	 Continue
c
	Ncd = Ncdn
c
	write(7,500)wbegin,wend,ncd,Nclass,factor
        write(7,100) 
        write(7,550)Proname,(ProCD(i),i=1,ncd)
550     format(//' SAMPLE CD: '//a80//(10f7.2))
c
c Read CD and STR information
c
	  iw1 = int(260.0 - wbegin) + 1
	  ind = 0
	  INBas = 0  
	  read(2,*)
	  read(2,*)
	  do i = 1,60
	    read(2,'(1x,a4)',end=99)IProt(ind+1)
	    read(2,104) (cdd(j),j=1,Ncd1)
	    read(1,*)
	    read(1,103) (sstr(j),j=1,Nclass)
	    INBas = INBas + 1  
	    rmsd = 0.0
	    do k=1,Ncd
	      rmsd = rmsd + (cdd(iw1+k-1)-ProCD(k))**2
	    enddo
	    rmsd = sqrt(rmsd/Ncd)
 	   If (rmsd.GT.0.01) then
c 	   If (rmsd.NE.0.0) then    ! Change Back Later
	    ind = ind + 1
	    k=0
	    do j=iw1,iw1+Ncd-1
	         k=k+1
	         BasCD(k,ind) = cdd(j)
	    enddo
	    do j=1,Nclass
	      Fsstr(ind,j) = sstr(j)
	    enddo
 	   Endif
	  Enddo
c
99	continue
      IF(NBasCD.EQ.0) NBasCD = INBas
        NBasCD = INBas
c
100     format(a50)
103	format(8f8.3)
104	format(15f6.2)
450     format(80('_'))
400     format(
     1 '   THE FRACTIONS OF SECONDARY STRUCTURES FROM',
     1/'   THE CDsstr Program (variable selection method)',
     2/'     Johnson W.C. Proteins:Struc.Func.Genet. 35, 307-312 (1999)',
     5/'     Program Modified by Sreerama. N.(Anal.Biochem.,287,252,'
     6,'2000)',/,'     For use with CDPro reference proteins')
501     format(/' Beginning wavelength:',f8.2,'; Ending wavelength:'
     1,f8.2,/' Total No of CD points',i4,/)
500     format(/' Beginning wavelength:',f8.2,'; Ending wavelength:'
     1,f8.2,/' Total No of CD points',i4,/
     1' Secondary structural elements=',i4,/' Multipilcation',
     1' FACTOR for CD spectrum=',f6.3)
470     format(//' The wavelength range: ',2f7.1,
     1   ', is different from that of',/,' the Database proteins:',2f7.1,
     2 /' The No. of CD points = ',i4, 
     3 //' The wavelengths are modified to be within the valid range')
540	format(/'  Proteins:',i3,';  CD data:', i3,';  Sec Str:',i3)
c	
        Nwave = Ncd
	NBasCD = Ind
c	write(7,500) Wbegin, Wend, Nwave
 	write(7,540) NBasCD, Ncd, NClass
	write(7,450)
	close(1)
	close(2)
      RETURN
999	continue
 	write(Nin,'(16h  All Done      )')
	STOP
      END
c
	Subroutine Fctrl(N,Fctr)
	Fctr = 1.0
	do i=1,N
	  Fctr = Fctr * float(i)
	enddo
c	write(6,*) N,Fctr
	end
c********************************************************************
c Subroutine to Calculate the Number of Segments
	Subroutine Segment(Str,MxSS,AHel,BStr,HelLn,BetLn)
	dimension Str(MxSS)
	AHel = Str(2) * 100.0 / 4.0
	BStr = Str(4) * 100.0 / 2.0
	If(AHel.LT.0.0) AHel = 0.0
	If(BStr.LT.0.0) BStr = 0.0
	HelLn = 0.0
	If(AHel.NE.0.0)HelLn = (Str(1) + Str(2)) * 100.0 /  AHel
	BetLn = 0.0
	If(BStr.NE.0.0)BetLn = (Str(3) + Str(4)) * 100.0 /  BStr    
	End
c ******************************************************************
c I/O
	Subroutine Message(N)
	If (N.EQ.1) write(*,9023)
9023	 format(80(1h*)
     1 /'         Program CDsstr--For analyzing Protein CD spectra'
     1 /'         Ref: W. Curtiss Johnson, Proteins (1999)' 
     2 /'         Adapted by N. Sreerama'
     3 /'         Input file: INPUT'
     4 /'         Output files: CDsstr.out, reconCD.out, ProtSS.out' 
     5 )              
	If (N.EQ.2)  then
		Write(*,9038)
 		read(*,*)
	endif
9038	format(80(1h_)//' End of Program'
     1 /6x,' Output in CDsstr.OUT; Final Result in ProtSS.out'
     2/6x,' Run SELCON3 and CONTINLL with this INPUT for Comparison'
     1 /6x,' For additional REFERENCE SETS change IBasis in INPUT'
     1/6x,' Result from different runs are in ProtSS.out'
     1/6x,' MULTIPLE sets of CD data can be analyzed:'
     1 /6x,'   Present multiple CD data sequentially in INPUT'
     2 /6x,' Hit Enter to CONTINUE')
	end

c*********************************************************************
c Subroutine to plot CD spectra
	Subroutine Plot(cd1,cd,wbegin,wend)
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	character*1 IPLT
	DIMENSION IPLT(21,60),cd(MxCD),cd1(MxCD),wave(MxCD)
	wavei=wend
	wavef=wbegin
	m=nint(wavef - wavei) + 1
	do 2 i= 1,m      
	  wave(i) = wavei + float(i) - 1
2       continue
	do 5 i=1,21
	 do 5 j=1,60
  	  iplt(i,j)=' '
      	  iplt(i,60)='|'
	  iplt(i,1)='|'
	  iplt(21,j)='-'
	  if (mod(j,6).eq.0) iplt(21,j)='+'
	  iplt(1,j)='-'
	  iplt(11,j)='-'
5	continue
	  iplt(21,1)='+'
	  iplt(11,1)='+'
	  iplt(11,60)='+'
	  iplt(1,1)='+'
	  iplt(1,60)='+'
c Determine Ymax Ymin
	cdmax= ymax(cd,cd1,m,MxCD)
	cdmin= ymin(cd,cd1,m,MxCD)
	facx=  float(wavef-wavei)/60.0
	ymx =amax1(cdmax,abs(cdmin))+0.5
	ymn = -ymx
	facy = ymx / 10.0
c	write(7,50)cdmin,cdmax,wavei,wavef,facx,facy,cdrms
	write(7,452)
50	format(/' YRANGE:',2f6.2,'; XRANGE:',2f5.0,'; FacX:',f5.3,
     1    '; FacY:',F6.3,/' RmsCD:',f5.3/)

	do 10 i=1,m    
452	format('          LEGEND: **** Calc. CD; oooo Expt. CD;',
     1   ' .... Diff. CD')
c
c Generate CD plot
c
	 x=(wave(i)-wavei)/facx
	 ix=nint(x)+1
	 y=cd(i)/facy
	 iy=nint(y)
	 iy=11-iy 
	 iplt(iy,ix)='*'
c
c Generate CD1 plot - SMPL
c
	 y=cd1(i)/facy
	 iy=nint(y)
	 iy=11-iy
	 iplt(iy,ix)='o'
c
c Generate CD1 - CD  Difference plot
c
	 y=(cd1(i)-CD(i))/facy
	 iy=nint(y)
	 iy=11-iy
	 iplt(iy,ix)='.'
c
10	continue
	write(7,250)ymx,(iplt(1,j),j=60,1,-1)
	write(7,300)((iplt(i,j),j=60,1,-1),i=2,20)
	write(7,250)ymn,(iplt(21,j),j=60,1,-1)
	Wintr = (wavef-wavei )/10.0
	write(7,350)wavei,Wintr,wavef
300	format(7x,60a1)
250	format(f7.0,60a1)
350	format(7x,f4.0,'  Wavelength at Intervals of:',f4.1,' nm '
     1        ,14x,f7.0//)
	end
c*********************888
	function ymax(a,a1,n,np)
	dimension a(np),a1(np)
	am=0.0
	do 10 i=1,n
	 if(a1(i).GT.am)am = a1(i)
10	 if(a(i).gt.am)am=a(i)
	ymax=am
	end
c************************
	function ymin(a,a1,n,np)
	dimension a(np),a1(np)
	am=0.0
	do 10 i=1,n
	 if(a1(i).lt.am)am=a1(i)
10	 if(a(i).lt.am)am=a(i)
	ymin=am
	end
c************************************************************************
	function rms_nrm(s1,s2,n1,np)
c  This function calculates the Normalized rms difference between S1 and S2
c  Wallace et.al. (2003) Prot. Sci., Mao and Wallace (1984)
c  S1 Experiment; S2 Calcuated
c
c NRMSD = Sum((Exp-Cal)**2) / Sum(Exp**2)
c
	dimension s1(np), s2(np)
	rms1=0.0
	Den = 0.0
	do  10 i=1,n1
	   rms1 = rms1 + (s1(i)-s2(i))**2
	   Den  = Den  + s1(i)**2
10      Continue  
crms = sqrt(rms1/float(n1))
	rms_nrm = sqrt(rms1/Den)
	end
