c SELCON3.FOR  --- SELCON, Self-Consistent method, Version 3
c
c -- Sample Input Files ---
c INPUT.SMP -- Sample of INPUT file. 
c
c -- Output Files ------
c OUTPUT.SMP -- Sample of OUTPUT file
c BASIS      -- Reference CD and SS data used is Printed
c
c***********************************************************************
c  Program to calculate the secondary structure content in a protein   *
c  given its CD spectrum. This is done using the CD spectra of a se    *
c  of proteins with known secondary structure content from x-ray       *
c  diffraction. The method is based on the Singular Value Decomposition*
c  of the  CD data matrix used with the secondary strcuture matrix.    *
c                                                                      *
c  The method of van Stokkum is used to implement the variable         *
c  selection method.                                                   *
c                                                                      *
c***********************************************************************
c    MxCD: Maximum number of CD data points - 83;                      *
c    MxPR: Max No. of proteins - 60                                    *
c    MxSS: Maximum Number of secondary structural elements - 6         *
c                                                                      *
c  Secondary Structre Elements are:                                    *
c   SSDATA.ext                                                         *
c   ext  --->  seg     pp2     ks4                                     *
c   MxSS  -->    6       5       4                                     *
c    Str1     Helix1   Helix   Helix                                   *
c    Str2     Helix2   Sheet   Sheet                                   *
c    Str3     Sheet1   Turns   Turns                                   *
c    Str4     Sheet2   PP2     Unord                                   *
c    Str5     Turns    Unord                                           *
c    Str6     Unord                                                    *
c                                                                      *
c    Helix: Alpha-helix and 310 helix combined                         *
c    Sheet: Beta Sheet(parallel and Anti-parallel combined),           *
c    PP2  : Poly(Pro)II type structure                                 *
c    Tyrns: Beta turns                                                 *
c    Unord: Unordered (also called as: random-coil, unassigned,        *
c                      unstructured, remainder etc)                    *
c                                                                      *
c  The fractions of Secondary structures were obtained from a combined *
c    analysis of PDB (x-ray) structure by DSSP (Kabsch & Sander) and   *
c    IPVAM (Sreerama & Woody; P2 structure indentifacation) programs   *
c    For segment analysis, H1-H2-S1-S2, see Prot.Sci, 1998 Paper.      *
c                                                                      *
c                                                                      *
c  References:                                                         *
c   SELCON: Sreerama & Woody, Anal Biochemistry, 1993,209, 32-44       *
c   PP2 str (VAMPS): Sreerama & Woody, Biochemistry,33,10022-25, 1994  *
c   Selcon Performance: Sreerama & Woody, J.Mol.Biol, 242,497-507,1994 *
c   Segment Analysis:Sreerama,Venyaminov and Woody,Prot.Sci.,8,370,1999*
c   CDPro Reference: Sreerama & Woody, Anal.Biochem.287,252-260,2000   *
c   Memb.Pro.Ref.  : Sreerama & Woody, Prot. Sci., 13,100-112,2004     *
c                                                                      *
c***********************************************************************

c**************************************************************************
c  Author: Narasimha SREERAMA, Colorado State University, Fort Collins
c          e-mail: sreeram@lamar.colostate.edu
c 
c
c  Acknowledgements:
c  Robert W. WOODY, Colorado State University, Fort Collins
c  Sergei Yu. Venyaminov, Mayo Clinic, Rochester
c  W. Curtiss Johnson, Oregon State University, Corvallis (For CD SPectra)
c
c**************************************************************************
c**********************************************************************

	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
        Dimension CDbas(MxCD,MxPR),StrBas(MxPr,MxSS),Smpl(MxCD),
     1        Smpl1(MxCD),NbasN(MxPR),RmsCD(MxPR)
	Character*4 title(16),NBasn,name,Print,StrN(6)
	Character*5 RefSet
c
c Number of Secondary Structures is SIX
c
c For Reference Protein Set 29 and 37, they are: 
c                    Helix1, Helix2, Strand1, Strand2, Turn and Other
c For Reference Set of 23 prtoteins they are
c                 Alpha-Helix, 310-Helix2, B-Sheet, Turn, PP2 and Other
c
c
	Call Message(1)
c
	NSec = 6
	Nin = 3
	Nout = 4
c Input
	OPEN(unit=Nin,file='INPUT')
 	OPEN(unit=Nout,file='SELCON3.OUT')
	OPEN(unit=1,file='BASIS')
	OPEN(unit=2,file='CalcCD.OUT')
	Idata = 0
1	CONTINUE
c
	IBasis = 0
	read(Nin,*,END=996)
	IData = IData + 1
	ISkip = 0
	write(Nout,450)
	write(*,450)
	write(*,'(/4x,9hCD DATA: ,i3)')IData
	write(Nout,400)
	read(Nin,*)
	read(Nin,*)IPrint,IBas1
	read(Nin,*)
	If(Iprint.EQ.1)Print='PRNT'
c
c Print OPTIONS
c
        write(Nout,982)
        write(Nout,983)
982     format(/6x,' BASIS SET OPTIONS AVAILABLE',
     1  /6x,'_________________________________________________________'
     1  /6x,'|                                                       |'
     1  /6x,'| IBasis Ref.Set  Proteins   Secondary Structre WaveLen |'
     1  /6x,'|    1   SP29   29-SOLUBLE     H1,H2,S1,S2,T,U  178-260 |'
     2  /6x,'|    2   SP22X  22-SOLUBLE     H,3-10,S,T,P2,U  178-260 |'
     3  /6x,'|    3   SP37   37-SOLUBLE     H1,H2,S1,S2,T,U  185-240 |'
     4  /6x,'|    4   SP43   43-SOLUBLE     H1,H2,S1,S2,T,U  190-240 |'
     5  /6x,'|    5   SP37A  37-SOLUBLE       H,S,P2,T,U     185-240 |')
983      format
     1  (6x,'|    6   SDP42  SP37,5 Denatrd  H1,H2,S1,S2,T,U 185-240 |'
     7  /6x,'|    7   SDP48  SP43,5 Denatrd  H1,H2,S1,S2,T,U 190-240 |'
     8  /6x,'|    8   CLSTR  Soluble/Denatr. H1,H2,S1,S2,T,U 190-240 |'
     9  /6x,'|    9   SMP50  SP37,13 Membran H1,H2,S1,S2,T,U 185-240 |'
     1  /6x,'|   10   SMP56  SP43,13 Membran H1,H2,S1,S2,T,U 190-240 |'
     1  /6x,'|_______________________________________________________|')
c
        write(Nout,901)IPrint,IBas1
901	format(3x,' IPrint  IBasis  ',/2x,7i8)
c Intialize Basis set
	StrN(1) = 'H(r)'
	StrN(2) = 'H(d)'
	StrN(3) = 'S(r)'
	StrN(4) = 'S(d)'
	StrN(5) = 'Trn '
	StrN(6) = 'Unrd'
c
c 29 Prot; 6 SS; Segment (178-260)
	If(IBas1.EQ.1) then
	    IBasis = 1
	    Iseg = 1
	    WvStrt = 260.0
	    WvEnd  = 178.0
	    RefSet = 'SP29 '
	Endif
c 23 Prot; 6 SS;                                (178-260)
	If(IBas2.EQ.1.OR.IBas1.EQ.2) then
	    IBasis = 2
	    Iseg = 0
	    WvStrt = 260.0
	    WvEnd  = 178.0
	    RefSet = 'SP23X'
	  StrN(1) = 'AHel'
	  StrN(2) = '3/10'
	  StrN(3) = 'BSht'
	  StrN(4) = 'Turn'
	  StrN(5) = 'PP2 '
	  StrN(6) = 'Unrd'
	Endif
c 37 Prot; 6 SS; Segment                        (185-240)
	If(IBas3.EQ.1.OR.IBas1.EQ.3) then   
	    IBasis = 3
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 185.0
	    RefSet = 'SP37 '
	Endif
c 43 Prot; 4 SS; (From Yang set)                (190-240)
	If(IBas3.EQ.2.OR.IBas1.EQ.4) then  
	    IBasis = 4
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 190.0
	    RefSet = 'SP43 '
	Endif
c 37 Prot; 5 SS; (For P2)                       (185-240)
	If(IBas2.EQ.2.OR.IBas1.EQ.5) then  
	    IBasis = 5
	    Iseg = 0
	    WvStrt = 240.0
	    WvEnd  = 185.0
	    NSec = 5
	    RefSet = 'SP37A'
	  StrN(1) = 'AHel'
	  StrN(2) = 'BSht'
	  StrN(3) = 'Turn'
	  StrN(4) = 'PP2 '
	  StrN(5) = 'Unrd'
	Endif
c 42 Prot; 6 SS; Segment; Denat                 (185-240)
	If(IBas3.EQ.3.OR.IBas1.EQ.6) then 
	    IBasis = 6
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 185.0
	    RefSet = 'SDP42'
	Endif
c 48 Prot; 6 SS; Segment;deant                  (190-240)
	If(IBas3.EQ.4.OR.IBas1.EQ.7) then 
	    IBasis = 7
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 190.0
	    RefSet = 'SDP48'
	Endif
c CA Prot; 6 SS; Segment;                        (190-240)
	If(IBas1.EQ.8) then 
	    IBasis = 8
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 190.0
	    RefSet = 'CLSTR'
	Endif
c 50 Prot; 6 SS; Segment;13MP+SP37               (185-240)
	If(IBas1.EQ.9) then 
	    IBasis = 9
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 185.0
	    RefSet = 'SMP50'
	Endif
c 56 Prot; 6 SS; Segment;13MP+SP42               (190-240)
	If(IBas1.EQ.10) then 
	    IBasis =10 
	    Iseg = 1
	    WvStrt = 240.0
	    WvEnd  = 190.0
	    RefSet = 'SMP56'
	Endif
c
	If(IBasis.EQ.1) then
	  write(Nout,781)RefSet
	  write(*,781)RefSet
781	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'29 Soluble Proteins',
     2          /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.2) then
	  write(Nout,782)RefSet
	  write(*,782)RefSet
782	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'23 Soluble Proteins; XTLSSTR structures',
     2          /'    Structures: A-Helix, 310-Helix, B-Sheet,',
     3          ' Turns, PP2 and Unordered'/)
	EndIf
	If(IBasis.EQ.3) then
	  write(Nout,783)RefSet
	  write(*,783)RefSet
783	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'37 Soluble Proteins',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.4) then
	  write(Nout,786)RefSet
	  write(*,786)RefSet
786	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'43 Soluble Proteins',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.5) then
	  write(Nout,787)RefSet
	  write(*,787)RefSet
787	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'37 Soluble Proteins; For P2 str. determination',
     2    /'    Structures: Helix, Sheet, Turns, P2, and Unordered'/)
	EndIf
	If(IBasis.EQ.6) then
	  write(Nout,788)RefSet
	  write(*,787)RefSet
788	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'42 Proteins - 37 Soluble and 5 Denatured',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.7) then
	  write(Nout,785)RefSet
	  write(*,785)RefSet
785	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'48 Proteins - 43 Soluble and 5 Denatured',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.8) then
	  write(Nout,789)RefSet
	  write(*,789)RefSet
789	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'Proteins belonging to a tertiary class',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.9) then
	  write(Nout,791)RefSet
	  write(*,791)RefSet
791	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'50 Proteins- 37 Soluble and 13 Membrane',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
	If(IBasis.EQ.10) then
	  write(Nout,792)RefSet
	  write(*,792)RefSet
792	  format(/'    Reference PROTEIN Set Selected: ',A5,/
     1           16x,'56 Proteins- 43 Soluble and 13 Membrane',
     2         /'    Structures: Helix1, Helix2, Strand1, Strand2,',
     3          ' Turns and Unordered'/
     4        16x,'Total Helix contet = Helix1 + Helix2'/
     5        16x,'Total Sheet contet = Strand1 + Strand2')
	EndIf
c
c
c
	If(ISeg.GE.1)
     1   write(Nout,'(/33h    NUMBER of Segments Calculated /)')
	write(Nout,784)
	write(*,784)
784	format(  '    SUMMARY in ProtSS.out'/
     2        '        Contains Sec. Str. Fractions, RMSD and NRMSD'
     3       /'        RMSD and Normalized-RMSD between Exp and Calc'
     4        ,' CD spectra'
     5      /'    NRMSD from the three programs generally vary in'
     6   ,' the order:'/'        CONTINLL < CDSSTR < SELCON3',
     7   /'    Use CAUTION in drawing CONCLUSIONS from RMSD/NRMSD'/)
c
	write(Nout,450)
	read(Nin,*)
        read(Nin,100,end=999) title
	read(Nin,*)
	write(Nout,100) 
	write(Nout,100) title
	write(*,100) title
	Const1 = 0.05
	Const2 = -0.03
	RmsLim=0.25
	NItr=20
cName = 'TEST'
	Name = title(1)
	read(Nin,*)
	read(Nin,*)Wbegin,Wend, FACTOR
	read(Nin,*)
	NCD = INT(wbegin-wend) + 1
	NCDN = NCD
c
	if((wbegin.GT.WvStrt).OR.(wend.LT.WvEnd))   then
		write(Nout,470) wbegin,wend,WvStrt, WvEnd, ncd
	   	Iskip = Wbegin - WvStrt
		if(Wbegin.GT.WvStrt) Wbegin = WvStrt
		if(Wend.LT.WvEnd) Wend = WvEnd
	      	NCDN = Int(Wbegin - Wend) + 1
		write(Nout,501)wbegin,wend,ncdn
c		STOP
	endif
	write(*,501)wbegin,wend,ncdn
c
	read(Nin,*)
	read(Nin,*,END=999,ERR=999)(smpl1(i),i=1,ncd)
	read(Nin,*)
	read(Nin,*)
c
	if(factor.EQ.0.0) Factor = 1.0
	 do 51 i=1,Ncdn
		smpl(i)=smpl1(i+Iskip)*factor
51	 Continue
	ncd = ncdn
c
	write(Nout,500)wbegin,wend,ncd,nsec,factor
	write(Nout,100) 
	write(Nout,550)(Title(i),i=1,10),(smpl(i),i=1,ncd)
c
	CALL SelBas(Name,Smpl,Ncd,Nsec,Nbas,NBasn,
     1       Wbegin,CDbas,StrBas,RmsCD,IBasis)
	CALL Order(Nbas,Ncd,Nsec,NBasn,CDbas,StrBas,RmsCD,Title)
	write(Nout,450)
	CALL GetStr(Nbas,Ncd,Nsec,Smpl,Name,Const1,Const2,
     1       WBegin,WEnd,CDbas,StrBas,RmsLim,Nitr,Print,
     2       ISeg,IBasis,StrN,Title,RefSet)
	write(Nout,450)
c
100     format(20a4)
450     format(80('_'))
550     format(//' SAMPLE CD: Protein - ',10a4,//(10f7.2))
400     format(
     1' THE FRACTIONS OF SECONDARY STRUCTURES',
     1' FROM THE SELF-CONSISTENT METHOD',
     2/'    Sreerama and Woody, Anal. Biochem. (1993), 209, 32',
     3/'    Sreerama and Woody, Biochemistry, 33, 10022-25, (1994),'
     4/'    Sreerama et al. Protein  Science, 8, 370-380, (1999),'
     5/'    Johnson W.C.Jr.,Proteins:Str.Func.Genet.35,307-312(1999)')
501     format(/' Beginning wavelength:',f8.2,'; Ending wavelength:'
     1,f8.2,/' Total No of CD points',i4,/)
500     format(/' Beginning wavelength:',f8.2,'; Ending wavelength:'
     1,f8.2,/' Total No of CD points',i4,/
     1' Secondary structural elements=',i4,/' Multipilcation',
     1' FACTOR for CD spectrum=',f6.3)
470     format(//' The wavelength range: ',2f7.1,
     1   ', is different from that of',/,' the Database proteins:',2f7.1,
     2 /' The No. of CD points = ',i4, 
     3 //' The wavelengths are modified to be within the valid range')
c
c
	Call Message(2)
c

	GOTO 1
c
996	Continue
	Write(*,
     1   '(/5x,18h *****************,
     2     /5x,18h *   ALL DONE    *,
     3     /5x,18h *****************)')
	CLOSE(1)
	CLOSE(2)
	CLOSE(Nin)
	CLOSE(Nout)
	STOP 
999     Continue
	Write(Nout,9001)
	Write(*,9001)
9001	format(//'  ERROR in file DATA. Insufficient CD Data'/)
	STOP
c
	END
c
c******************************************************************************
c
	Subroutine SelBas(Name,Smpl,Ncd1,Nsec,Nbas,NBasn,
     1       Begwav,CDbas,StrBas,RmsCD,IBasis)
c
c   this subroutine reads from the CD database the CD spectra of the 
c   basis protiens and calculates the RMS difference between the SMPL
c   and the PROTCD, which is stored in RMSCD.
c   Calls routine RMS
c
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	Parameter (Nin = 3, Nout =4)
c
	character*4 namep,namep1,nbasn,name
        Dimension CDbas(MxCD,MxPR),StrBas(MxPr,MxSS),Smpl(MxCD)
	dimension prtncd(MxCD),nbasn(MxCD),prtstr(MxSS),
     1   temp(MxCD),RmsCD(MxPR)
c
	Ncd = MxCD
	WavPrt = NCDBeg
	iwvst = int(wavprt - begwav)
	If(IBasis.EQ.1) then
	  OPEN (unit=8,file='CDDATA.29')
	  OPEN (unit=9,file='SSDATA.29')
	EndIf
	If(IBasis.EQ.2) then
	  OPEN (unit=8,file='CDDATA.23')
	  OPEN (unit=9,file='SSDATA.23')
	EndIf
	If(IBasis.EQ.3) then
	  OPEN (unit=8,file='CDDATA.37')
	  OPEN (unit=9,file='SSDATA.37')
	EndIf
	If(IBasis.EQ.4) then
	  OPEN (unit=8,file='CDDATA.43')
	  OPEN (unit=9,file='SSDATA.43')
	EndIf
	If(IBasis.EQ.5) then
	  OPEN (unit=8,file='CDDATA.P2')
	  OPEN (unit=9,file='SSDATA.P2')
	EndIf
	If(IBasis.EQ.6) then
	  OPEN (unit=8,file='CDDATA.42')
	  OPEN (unit=9,file='SSDATA.42')
	EndIf
	If(IBasis.EQ.7) then
	  OPEN (unit=8,file='CDDATA.48')
	  OPEN (unit=9,file='SSDATA.48')
	EndIf
	If(IBasis.EQ.8) then
	  OPEN (unit=8,file='CDDATA.CA')
	  OPEN (unit=9,file='SSDATA.CA')
	EndIf
	If(IBasis.EQ.9) then
	  OPEN (unit=8,file='CDDATA.50')
	  OPEN (unit=9,file='SSDATA.50')
	EndIf
	If(IBasis.EQ.10) then
	  OPEN (unit=8,file='CDDATA.56')
	  OPEN (unit=9,file='SSDATA.56')
	EndIf

	rewind (8)
	rewind(9)
	read(8,100)
	read(8,100)
	Nbas = 1
	Nbasn(1) = name
	RmsCD(1) =0.0
c
	Do 12 i=1,Ncd1
	    cdbas(i,1) = smpl (i)
12      Continue  
c
   1    continue
c
	read (8,110,end=99) namep,factor
	write(1,'(2x,a4)') namep
c
c   factor to multiply the CD spectrum of the basis set protein
c
	read(8,115)(prtncd(i),i=1,ncd)
	if(factor.NE.0.0) then
	  do 14 i=1,ncd
 	     prtncd(i)=prtncd(i)*factor
14        Continue  
	endif
	read(9,110)namep1
	if(namep.ne.namep1) write(Nout,921)namep,namep1
	read(9,120) (prtstr(i),i=1,6)
	icd=1
	do 16 i=iwvst+1,ncd
	    temp(icd) = prtncd(i)
	    icd=icd+1
16      Continue  
	delcd = rms (smpl,temp,ncd1,MxCD)
c
	if(delcd.LT.0.01) GOTO 1  
c	if(delcd.EQ.0.0) GOTO 1   ! Change back LATER
c
	nbas = nbas + 1
	IF (nbas.GT.MxPR) then
	   Write(Nout,301)
	   Write(*,301)
301	   format(' Number of Proteins in the database is > MxPR',
     1	          ' For the program to work increase the dimension')
	   STOP
	ENDIF
c
	NBasn(nbas) = namep
	RmsCD (nbas) = delcd
	do 22 i=1,Ncd1
           CDbas(i,nbas) = temp(i)
22      Continue  
	do 24 i=1,Nsec
           StrBas (nbas, i) = prtstr(i)
24      Continue  
	go to 1
c   
c
  99    continue
c
c   output
c
	write(1,405) nbas, (nbasn(i), i=1,nbas)
	write (1,410) nsec
	do 32 i=1,Nbas
           write (1,'(i4,6f8.3)') i,(StrBas(i,j),j=1,Nsec)
32      Continue  
	write(1,411) 
        write (1,'(8i5)') (int(begwav)+1-i, i=1,ncd1)
	write(1,412) 
	do 34 i=1,Nbas
           write (1,'(i4/(10f8.2))')i,(cdbas(j,i), j=1,ncd1)
34      Continue  
c
100     format(i5)
101     format(6x,6f8.3)
105     format(12(2x,a4))
110     format(1x,a4,f6.3)
115     format(15f6.2)
120     format(6f8.3)
405     format('  Nbas=',i4,/'  Prtn :'/,(12(2x,a4)))
410     format('  No of sec str elements=',i5)
411     format('  Wavelength range')
412     format('  CD basis')
921     format(' NAMES OF PROTEINS IN CDDATA:',a4,' AND SSDATA:',a4,
     1  ' DONOT MATCH !')
	close(8)
	close(9)
c
	RETURN
	END
c
c*****************************************************************************
c
	Subroutine Order(Nbas,Ncd,Nsec,NBasn,CDbas,StrBas,RmsCD,Title)
c
c  this routine arranges the RMSCD values in the increasing ordr.
c  This is needed to ordr the basis CD in the increasing ordr of
c  delta(c) (von Stokkum et. al.), so as to select the best basis.
c
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	Parameter (Nin = 3, Nout =4)
c
	character*4 nbasn,nmo,Title(16)
	dimension Nbasn(MxPR),prtstr(MxSS),CDbas(MxCD,MxPR),
     1     StrBas(MxPR,MxSS),RmsCD(MxPR),nu(MxPR), ordr(MxPR), 
     2     cdord(MxCD,MxPR), strord(MxPR,MxSS),nmo(MxPR)
c
	do 11 j=1,NBas
	  ordr(j)= RmsCD(j)
	  nu(j)=j
	  do 12 i= 2,NBas
	    if (ordr(j).gt.RmsCD(i)) then
	       ordr(j)=RmsCD(i)
	       nu(j) = i
	    endif
12        Continue  
	  RmsCD(nu(j)) = 999.0
11      Continue  
c
c  Now NU() contains the ordr of RMSCD(), the number I. And ORDER
c  contains the new delta(c) ordred. 
c  Now ordr the CDBAS and STRBAS in the delta(c) ordr.
c
	do 21 j=1,NBas
	   nmo(j)=NBasn(nu(j))
	   RmsCD(j)=ordr(j)
	   do 22 i=1,ncd
             cdord(i,j)=CDbas(i,nu(j))
22         Continue  
	   do 23 i=1,nsec
              StrOrd(j,i) = StrBas(nu(j),i)
23         Continue  
21      Continue  
c
	do 31 i=1,NBas
	  NBasn(i)=nmo(i)
	  do 32 j=1,nsec
	     StrBas(i,j)=StrOrd(i,j)
32        Continue  
	  do 33 j=1,ncd
	     cdbas(j,i)=cdord(j,i)
33        Continue  
31      Continue  
c
c READ initial guess; if iguess = 0 select initial guess
c Copy STRBAS(2,i) to STRBAS(1,i) ; To make the initial guess
c equal to the structure with closest CD Spectrum.
c
	iguess = 0
	read(Nin,101)iguess,(prtstr(k),k=1,nsec)
	if (iguess.EQ.0) then
	   do 41 i=1,nsec
	      StrBas(1,i) = StrBas(2,i)
41         Continue  
	 else
	  do 42 i=1,nsec
		   StrBas(1,i) = prtstr(i)  
42        Continue  
	endif

	write(Nout,405) NBas, (title(i),i=1,10)
	write (Nout,400) (RmsCD(i),i=1,NBas)
	write(Nout,403) (NBasn(i), i=1,NBas)
	if(iguess.EQ.0) then
		write(Nout,422)NBasn(2)
	  else
		write(Nout,423)
	endif
c
	write(Nout,421)(StrBas(1,i),i=1,nsec)
c       write (Nout,410) nsec
c       do  i=1,nsec
c         write (Nout,415)i, (StrBas(i,j),j=1,NBas)
c       enddo
c       do  i=1,ncd
c         write (Nout,415) i,(cdbas(i,j), j=1,NBas)
c       enddo
c
101     format(i4,6f8.3)
400     format(//'  Ordered DELTA(CD) values :',/(2x, 8f8.4))
403     format(//'  Ordered List of PROTEINS :',/(8(4x,a4)))
405     format(/' Number of Proteins in the database=',i4,//
     1' The SAMPLE data for:  ',10a4)
410     format('  no of sec str elements=',i5)
415     format(i4,20f6.2)
421     format(' Initial Guess:', 6f8.3)
422     format(//' IGUESS = 0; The structure of the Protein with',
     1' closest CD spectrum:',1x,a4/)
423     format(//' IGUESS = 1; User supplied Initial Guess'/)
	return
	end
c************************************************************************
	function rms(s1,s2,n1,np)
c  This function calculates the rms difference between S1 and S2
	dimension s1(np), s2(np)
	rms1=0.0
	do  10 i=1,n1
	   rms1 = rms1 + (s1(i)-s2(i))**2
10      Continue  
	rms = sqrt(rms1/float(n1))
	end
c****************************************************************************

	Subroutine Eigen(np,ncd,s,u,v)
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	Parameter (Nin = 3, Nout =4)
c
c  This routine orders the eigen values, diagonal elements
c  of the S matrix in R1, in the decreasing order. It also separates out
c  the eigen vectors from the unitary matrices U and V in the same order.
c
	dimension  u(MxCD,MxPR), v(MxPR,MxPR), s(MxPR)
	dimension  nu(MxPR), stmp(MxPR), utmp(MxCD,MxPR), vtmp(MxPR,MxPR)
     
c
c  find the highest diagonal elements
c
	do 12 j=1,np
	   stmp(j)=s(1)
	   nu(j)=1
	   do 14 i=1,np
	     if (s(i).GT.stmp(j)) then
	       stmp(j)=s(i)
	       nu(j)=i
	     endif
14         Continue  
	   s(nu(j))=0.0
	   do 16 i=1,ncd
	      Utmp(i,j) = U(i,nu(j))
16         Continue  
	   do 18 i=1,np
	      Vtmp(i,j) = V(i,nu(j))
18         Continue  
12      Continue  
c
c Copy Stmp to S
c
	do 22 i=1,np
	   s(i) = stmp(nu(i))
22      Continue  
c
c  Copy the eigen vectors corresponding to the highest eigen values 
c
	do 32 j=1,np
	  do 34 i=1,ncd
	     U(i,j) = Utmp(i,j)
34        Continue  
	  do 36 i=1,np
	     V(i,j) = Vtmp(i,j)
36        Continue  
32      Continue  
c
	return
	end
c***********************************************************************
	Subroutine GetStr(Nbas,Ncd,Nsec,Smpl,NameP,Const1,Const2,
     1   WBegin,WEnd,CDbas,StrBas,RmsLmt,NItr,Print,ISeg,IBasis,StrN,
     2   Title,RefSet)
c
c this routine gets the structure as a variation of NBAS and NS (diag eles)
c
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	Parameter (Nin = 3, Nout =4)
c
	character*4 NameP,Print,StrN(6),SumN,RMSN1,RMSN2,Title(16)
     1      ,IProt
	Character*5 RefSet
	dimension U(MxCD,MxPR),V(MxPr,MxPR),CDbas(MxCD,MxPR),
     1            StrBas(MxPR,MxSS),X(MxPR),rv1(MxPR),S(MxPR)
	dimension Smpl(MxCD),Str(MxSS),Guess(MxSS)
        dimension RecCD(MxCD,MxPR),RecCDS(MxCD),CalCD(MxCD)
	dimension SF(MxSS),UF(MxPR,MxSS),VF(MxSS,MxSS)
	dimension RmsCol(MxSol),StrCol(MxSol,MxSS),CDSCol(MxSol,MxCD),
     1            RmsRCN(MxSol),RmsEXP(MxSol)
	dimension IDSol(MxSol),IndSol(MxSol),IBas(MxSol,2)
c
c Convergence value for Iterations: 0.005    -- 1% diff between Guesses
c
	CNVRG = 0.005
	SumN = 'SUM '
	RMSN1 = 'RRcn'
	RMSN2 = 'RExp'
c
c----------------------------------------------------
c Determine Helix fraction from HJ method. Nbas, Ns=5
c
         call SVD(NCD,NBas,CDbas,S,U,V,ierr,rv1,MxCD,MxPR)
         call Eigen(NBas,ncd,S,U,V)
c
	 do 12 ip = 1,NBas
	       X(ip) = 0.0
12       Continue  
	 Do 21 is = 1,5
	   Prodct = 0.0
	   do 22 icd = 1,ncd
	      prodct = prodct + U(icd,is) * Smpl(icd)
22         Continue  
	   if (S(is).NE.0.0) prodct = prodct / S(is)
	   do 23 ip = 1,NBas
	       X(ip) = X(ip) +  V(ip,is) * Prodct
23         Continue  
21       Continue  
c
c Iterate for Self-consistent solution for Basis: NBas,NS -- i
c 10 Iterations are more than sufficient.
c
	 Do 32 Itr = 1,10
	   SumSS = 0.0
	   do 34 iss = 1,Nsec
	     Str(iss) = 0.0
	     do 33 ip = 1,Nbas
	       Str(iss) = Str(iss) + X(ip) * StrBas(ip,iss)
33           Continue  
c	     if(Str(iss).GT.0.0) SumSS = SumSS + Str(iss)
 	     SumSS = SumSS + Str(iss)
	     StrBas(1,iss) = Str(iss)
	     if(Str(iss).LT.0.0) StrBas(1,iss) = 0.0
34        Continue  
32      Continue  
	Write(Nout,1451) (StrN(k),k=1,nsec),SumN
	Write(Nout,145) (Str(k),k=1,nsec),SumSS
1451	format(/' Solution Corresponding to All proteins in the Basis',
     1          ' and 5 SVD vectors'/' This corresponds to Hennessey',
     2          ' and Johnson Method',//5x,7(3x,a4))
145 	format(5x,7f7.3)
c
c Determine HelHJ. For Segment analysis Str1+Str2; for others Str1
c
	HelHJ = Str(1)
        If(IBasis.NE.2.AND.IBasis.NE.5)
     1      HelHJ = Str(1)+Str(2)
	Write(Nout,'(/25H Helix From H & J Method: ,7f8.3/)') HelHJ
	write(Nout,450)
450     format(/80('_')/)
c
c-----------------------------------------------------------------
c  Start Calculation
c  Number of proteins  in the Basis set is Varied from  7 to  NBas
c  Make initial Guess -- May need to be altered based on HJHel!
c
	do 41 k=1,Nsec
	   Guess(k) = StrBas(2,k)
41      Continue  
	RmsGss = 1.00
	Write(Nout,320)NItr-1
320	format(/' FIRST STAGE:',
     1         /' Solutions With FIRST TWO Selection Rules:'/
     2         ,' This is for the First',I3,' Iterations only'/
     3         ,' Constraints relaxed to get a MIN # of Solns'//)
c	Write(Nout,330)
1296	format('NSol NBas Ns',2x,a4,6(3x,a4))
c
c Iterate for Self-Consistent Solution
c Corresponds to SELCON (Original Method)
c
	DO 92 Itr = 1,NItr-1               
	   if(RmsGss.LT.CNVRG) then
              Write(Nout,'(/17h SOLN. CONVERGED:,i3,11h ITERATIONS/)')Itr-1
	      write(Nout,'(/8h MinSol:,i3,7h  SUM <,f7.3)')MinSol,SumLim
	      write(Nout,5451)(StrN(k),k=1,Nsec),SumN
	      write(Nout,545)(Guess(k),k=1,Nsec),SumSS
5451	      format(/'  Solution at the END of FIRST STAGE:',//
     1              3x,7(3x,a4))
545           format(3x,7f7.3)
 	       GoTo 99
	   else
	       if(Print.EQ.'PRNT')Write(Nout,'(/11H ITERATION:,i4/)')Itr
	   endif
	  do 44 k=1,Nsec
	   StrBas(1,k) = Guess(k)
44        Continue  
	  NSoln = 0
	  MinSol = 1
	  SumLim = Const1
	  FrcLim = Const2
	  RmsLim = RmsLmt
1101	  Continue
c
c  NBAS LOOP
	  DO 94 J= 6,Nbas                 
c
c  Call SVD to create U, S and V matrices for CD(ncd,j) 
c  Once for the J - LOOP
c
c  NS - # of SVD vectors - varied from 1 to 8 or 1 to J-2, for J < 9
c
c  Str = StrBas * V * S(inverse) * U(transpose) * SmplCD 
c
           call SVD(Ncd,j,CDbas,S,U,V,ierr,rv1,MxCD,MxPR)
           call Eigen(j,ncd,S,U,V)
c  NS LOOP
	   Do  96 I = 1,8     
	   IF (i.LT.(j-2)) then   
 	     NS = i
	     do 46 ip = 1,J
	        X(ip) = 0.0
46           Continue  
	     Do 48 is = 1,NS
	        Prodct = 0.0
		do 52 icd = 1,ncd
		   prodct = prodct + U(icd,is) * Smpl(icd)
52              Continue  
		if (S(is).NE.0.0) prodct = prodct / S(is)
		do 54 ip = 1,J
		   X(ip) = X(ip) +  V(ip,is) * Prodct
54              Continue  
48           Continue  
c Calculate Structure for Basis: J,NS
		SumSS = 0.0
	        do 56 iss = 1,Nsec
		   Str(iss) = 0.0
		   do 58 ip = 1,J
		      Str(iss) = Str(iss) + X(ip) * StrBas(ip,iss)
58      	   Continue  
c		   if(Str(iss).GT.0.0) SumSS = SumSS + Str(iss)
 	           SumSS = SumSS + Str(iss)
56      	Continue  
c Collect Solns During Iterations < NITR
		Nsoln = Nsoln + 1
	  	IBas(Nsoln,1) = J
	  	IBas(Nsoln,2) = NS
	 	do 62 k=1,nsec
		   StrCol(NSoln,k) = Str(k)
62       	Continue  
c 	       Write(Nout,1610)NameP,J,NS,(Str(k),k=1,nsec),SumSS
	  EndIF
96        Continue  
c NS LOOP
94      Continue  
c NBAS LOOP
	If(Itr.EQ.1)Write(Nout,'(/5x,20h TOTAL LL SOLUTIONS:,I4)')Nsoln
c
c Select Solutions Based on the First TWO selection rules
c
	IsolP = Isol
	Isol = 0
1291	Continue
	If(SumLim.GE.0.14) then
	 if(Isol.EQ.0.AND.MinSol.EQ.1) then
	  If(Itr.EQ.2.AND.IsolP.EQ.0) then
	    Write(Nout,1295) Itr,(Guess(k),k=1,Nsec)
1295	    format(//' *** SELCON FAILED ***'//
     1     '  NO SOLUTIONS are obtained even with 0.85 < SumSS ',
     2     ' < 1.15; 15% error'//'  CHECK DATA'//
     3     '    RESULT at ITER:',i3,' Str:',7f8.3)
	    Write(Nout,1195)NameP,(Smpl(k),k=1,Ncd)
1195	    format(//'  CD Spectra should be in the range:'/
     1    '   -10.0 to +20.0 Delta(Epsilon) Units.'//
     2    '  PROTEIN:',2x,A4,/'  CD SPECTRA:'/(10f8.3))
	    GO TO 999
	  Endif
	  write(Nout,1281)Itr
1281	  format(//' *** SELCON PARTIALLY COMPLETED ***'//,
     1    ' Convergence Of Successive Solutions Was Not Achieved'//,
     2    ' No Solutions were Obtained AT ITERATION:',I3)
	  Isol = IsolP
	  write(Nout,1197)Isol,Itr-1
1197	  format(//i4,' Solutions From Iteration:',I3,' is Used to',
     1    ' CONTINUE to SECOND STAGE.',/'   For Details Use PRTN ',
     2    ' Option to print Intermediate results')
	  GoTO 99
	 else
	  IF(MinSol.GT.1)MinSol = MinSol - 1
	  if(Print.EQ.'PRNT') write(Nout,1297)MinSol,SumLim,FrcLim
	  if(Print.EQ.'PRNT') Write(Nout,1296) (StrN(k),k=1,Nsec),Sumn
	 endif
	 Else
	  if(Print.EQ.'PRNT') write(Nout,1297)MinSol,SumLim,FrcLim
	  if(Print.EQ.'PRNT') Write(Nout,1296) (StrN(k),k=1,Nsec),Sumn
	 EndIf
1297	 format(/'  TotSOL >',I3/,'  Limits: ABS(Sum-1.0) <',f7.3,
     1    '; Each Fraction >',f7.3)
         Isol = 0
	 do 64 k =1,Nsec
	  str(k) = 0.0
64       Continue  
	 Do 68 is = 1,NSoln
	   SumSS = 0.0
	   do 72 k = 1,Nsec
	        if(StrCol(is,k).LT.FrcLim) GOTO 1299
c	        if(StrCol(is,k).GT.0.0) SumSS = SumSS + StrCol(is,k)
 	        SumSS = SumSS + StrCol(is,k)
72         Continue  
	   If(abs(SumSS-1.0).GT.SumLim) GOTO 1299  
	   Isol = Isol + 1
	   IDSol(Isol) = is
	   do 74 k=1,nsec
	      Str(k) = Str(k) + StrCol(is,k) 
74         Continue  
	   if(Print.EQ.'PRNT') Write(Nout,'(2I4,i3,7f7.3)') 
     1       is,IBas(is,1),IBas(is,2),(StrCol(is,k),k=1,nsec),SumSS
1299       Continue
68       Continue  
c
	 If(Isol.GE.MinSol) then
	   SumSS = 0.0
	   do 76 k=1,nsec
	      Guess(k) = Str(k) / Isol
c	      if(Guess(k).GT.0.0) SumSS = SumSS + Guess(k)
 	      SumSS = SumSS + Guess(k)
	      Str(k) = StrBas(1,k)
76         Continue  
	   RmsGss = RMS(Guess,Str,Nsec,MxSS)
	   Write(Nout,'(/6h ITER:,i2,9h; AVE OF ,I3,6h SOLN:,6f7.3)')
     1       Itr,Isol,(Guess(k),k=1,nsec)
      	   Write(Nout,'(26h SUM of SECNDRY STRUCTURE: ,f7.3)')SumSS
	   Write(Nout,'(26h RMSD with Previous Guess:,f8.4)')RmsGss
	 Else
	   if (SumLim.LT.0.14) then
	     SumLim = SumLim + 0.02
	     FrcLim = FrcLim - 0.01
	   endif
	   GoTo 1291
	 EndIf
c
92      Continue  
 1610	format(1x,i4,i4,i3,9f7.3)
c
c---------------------------------------
99	CONTINUE
c 
	 Write(Nout,444)
c 	 Write(7,'(1x,A4,7f8.3)')NameP,(Guess(k),k=1,nsec),SumSS
444	 format(//' First Part Completed.  The results roughly',
     1    ' correspond',
     1    / ' to SELCON and SELCON1 with only 2 selection rules')
	 write(Nout,450)
	 write(Nout,1393)
1393	   format('  SECOND STAGE:'/ '  SOLUTIONS THAT SATISLY ',
     1       'THREE SELECTION RULES:'//)
1293       format('   I NSol Bas NS',9(3x,a4))
c
c  Calculate the CD spectra 
c
c  RecCD = U * S * V(transpose); 
c  Reconstruct CD spectral matrix corresponding to Nbas = J and NS SVD cofs
c  RecCDS:  Reconstructred CD spectrum of the Sample
c
c
	Nsol = Isol
	DO 98 i1=1,Nsol
	   ID = IDSol(i1)
	   Do 78 icd = 1,ncd
	     do 79 ip = 1,IBas(ID,1)
		RecCD(icd,ip) = 0.0
	        do 80 is = 1,IBas(ID,2)
		   RecCD(icd,ip) = RecCD(icd,ip) + 
     1                             U(icd,is) * S(is) * V(ip,is)
80      	Continue  
	         If(ip.EQ.1)RecCDS (icd) = RecCD(icd,1)
79            Continue  
78         Continue  
c
c Compute the CD spectrum of the protein corresponding to the structure Str
c   c = f * X ------>  C(transpose) =  F(transpose) * X Matrix
c                       (NBas,NCD)     (Nsec,NBas)  * (Nbas,NCD)
c    X is determined using SVD on F(transpose) matrix and matrix multiplic.
c
c CalCD = Str * VF * SF(inverse) * UF(transpose) * RecCD
c 
            call SVD(IBas(ID,1),Nsec,StrBas,SF,UF,VF,ierr,rv1,MxPR,MxSS)
c
	     do 82 ip = 1,IBas(ID,1)
	        X(ip) = 0.0
82           Continue  
	     Do 83 is = 1,NSec
	        Prodct = 0.0
		do 84 ic = 1,Nsec
		   prodct = prodct + VF(ic,is) * StrCol(ID,ic)
84      	Continue  
		if (SF(is).NE.0.0) prodct = prodct / SF(is)
		do 86 ip = 1,IBas(ID,1)
		   X(ip) = X(ip) +  UF(ip,is) * Prodct
86              Continue  
83           Continue  
	     do 87 icd = 1,Ncd
	        CalCD(icd) = 0.0
		do 88 ip = 1,IBas(ID,1)
		   CalCD(icd) = CalCD(icd) + X(ip) * RecCD(icd,ip)
88      	Continue  
87           Continue  
c
c RmsCDC: RMS Calc vs. Reconstr. CD
c RmsCDS: RMS Expt. vs. Reconstr. CD
c
	     RmsCDC = rms(RecCDS,CalCD,Ncd,MxCD)
	     RmsCDS = rms(smpl,RecCDS,Ncd,MxCD)
	     RmsRCN(ID) = RMSCDS
	     RmsEXP(ID) = RMSCDC
	     RmsCol(ID) = RMSCDC
	     If(RmsCDC.GT.RmsCDS) RmsCol(ID) = RMSCDS
	     do 89 k=1,NCD
		CDSCol(ID,k) = CalCD(k)
89           Continue  
c  	     If(Print.EQ.'PRNT')Write(Nout,1610)ID,IBas(ID,1),IBas(ID,2)
c     1       ,(StrCol(ID,k),k=1,nsec),SumSS,RmsCol(ID)
98      Continue  
c
	  Write(Nout,1293) (StrN(k),k=1,Nsec),SumN,RMSN1,RMSN2
1391	  Continue
          Isol = 0
	  do 122 k =1,Nsec
	     str(k) = 0.0
122       Continue  
          do 123 k=1,Ncd
	     CalCD(k) = 0.0   
123       Continue  
	  If (RmsLim.GT.0.60) then
	    If(Isol.EQ.0.AND.MinSol.EQ.1) then
	      Write(Nout,1395) RmsLim,Itr,(Guess(k),k=1,Nsec)
1395	      format(' SELCON FAILED AT SECOND STAGE: '/
     1        '  NO SOLUTIONS OBTAINED WITH RmsCD <',f7.3,
     2        /'  ITER=', i3,'  STR:',7f8.3)
	      GOTO 999
	    else
	      If(MinSol.GT.1)MinSol = MinSol - 1
	      if(Print.EQ.'PRNT') write(Nout,1298)MinSol,RmsLim
	      if(Print.EQ.'PRNT') 
     1	         Write(Nout,1293) (StrN(k),k=1,Nsec),SumN,RMSN1,RMSN2
	    endif
	   Else
	      if(Print.EQ.'PRNT') write(Nout,1298)MinSol,RmsLim
	      if(Print.EQ.'PRNT') 
     1	         Write(Nout,1293) (StrN(k),k=1,Nsec),SumN,RMSN1,RMSN2
	  EndIf
1298     format(/' TotSOL >',I3,'; Limits: RmsLim  <',f7.3)
1398	 format(/'  TotSOL >',I3/,'  Limits: ABS(Sum-1.0) <',f7.3,
     1    '; Each Fraction >',f7.3/'  RmsCD(Exp,Cal) <',f7.3)
c COMPARE RMSCD RMSLIM
	 Do 127 is = 1,NSol
	   ID = IDSol(is)
	   SumSS = 0.0
	   do 128 k = 1,Nsec
c	        if(StrCol(ID,k).GT.0.0) SumSS = SumSS + StrCol(ID,k)
 	        SumSS = SumSS + StrCol(ID,k)
128        Continue  
 	   If(RmsCol(ID).GT.RmsLim) GOTO 1399
	   Isol = Isol + 1
	   IndSol(Isol) = ID
	   do 129 k=1,nsec
	      Str(k) = Str(k) + StrCol(ID,k) 
129        Continue  
	   do 131 k=1,ncd
	      CalCD(k) = CalCD(k) + CDSCol(ID,k)
131        Continue  
 	   Write(Nout,'(3i4,i3,10f7.3)')Isol,ID,Ibas(ID,1),IBas(ID,2),
     1              (StrCol(ID,k),k=1,nsec),SumSS,RmsRCN(ID),RmsEXP(ID)
1399       Continue
127       Continue  
c	If(Isol.GT.MinSol) then
 	If(Isol.GE.MinSol) then
	   SumSS = 0.0
	   do 132 k=1,nsec
	      Guess(k) = Str(k) / Isol
c	      if(Guess(k).GT.0.0) SumSS = SumSS + Guess(k)
 	      SumSS = SumSS + Guess(k)
132        Continue  
           do 134 k=1,Ncd
	     CalCD(k) = CalCD(k) / Isol   
134        Continue  
	   RmsCal = rms(smpl,CalCD,Ncd,MxCD)
	   write(Nout,1398)MinSol,SumLim,FrcLim,RmsLim
	   Write(Nout,1397) Isol,(StrN(k),k=1,nsec),SumN,RmsN1
	   Write(Nout,1396) (Guess(k),k=1,nsec),SumSS,RmsCal
c	   Backspace(7)
c 	   Write(7,'(1x,A4,7f8.3)')NameP,(Guess(k),k=1,nsec),SumSS
1397	   format(/'  SOLUTION AT SECOND STAGE (SELCON2):'/,
     1      '  Average Solution From',I4,'  Solutions:',//
     2      8(3x,a4))
1396       format(8f7.3)
	   write(Nout,447)
447	 format(/' Second Part Completed.  The results roughly ',
     1    /' correspond SELCON2 with three selection rules')
	  If(Iseg.NE.0) then
	     CALL Segment(Guess,MxSS,AHel,BStr,HelLn,BetLn)
	     write(Nout,452)AHel,BStr
	     write(Nout,453)
	     write(Nout,456)HelLn,BetLn
	  EndIf
453        format(//'  For YOUR PROTEIN multiply No. of SEGMENTS by'
     1     ,/'  (Number of Residues) / 100'/'  e.g., If Number of',
     2     ' Residues = 153,  Use a FACTOR of 1.53',//
     3     '  Average Length of Segments Remains as Estimated') 
452	 format(/5x,'+',50('-'),'+',/,5x,'|  Based on the SOLUTION',
     1     ' the Number of SEGMENTS of |',/,5x,'|   HELICES (Per 100',
     2     ' Residues) are:',f10.3,5x,' |',/5x,'|   STRANDS (Per 100',
     3     ' Residues) are:',f10.3,5x,' |',/5x,'+',50('-'),'+')
456	 format(/5x,'+',50('-'),'+',/,5x,'|  Based on the SOLUTION',
     1     ' Obtained                  |'
     2     /5x,'|   The AVERAGE LENGTH of HELICES :',f10.3,5x,' |',
     3     /5x,'|   The AVERAGE LENGTH of STRANDS :',f10.3,5x,' |',
     4     /5x,'+',50('-'),'+')
	   write(Nout,450)
	   write(Nout,724)Isol,NameP
724	 format(//'  COMPARISON OF EXPT. AND CALC. CD',
     1   ' SPECTRA (Ave. of ',i3,' Solns) OF: ',A4//)
	   CALL Plot(Smpl,CalCD,Wbegin,Wend)
 	   Rms_N = rms_nrm(smpl,CalCD,Ncd,MxCD)
	   Write(Nout,'(/31h NRMSD betwen Expt. & Calc. CD=,f7.3/)')Rms_N
	   write(Nout,450)
c	
c	rewind(2)
 	write(2,239)
 	WaveL = Wbegin
 	Do ic=1,ncd
 	   write(2,238)WaveL,Smpl(ic),CalCD(ic)
 	   WaveL = WaveL - 1
 	EndDo
c
	Else
	   if (RmsLim.LT.0.60) then
	     RmsLim = RmsLim + 0.10
	   endif
	   GoTo 1391
	EndIf
c
c---------------------------------------
c
c Now Use Rule 4 for Narrowing Solutions. 
c JOhnson's Method CDsstr gave results with remarkable accuracy with this rule
c    HelHJ > 0.65 -- all solutions with Helix > 0.65 are averaged
c  0.65 > HelHJ > 0.25 -- all solutions with Helix = (HelHJ+maxH) + or - 0.03
c  0.25 > HelHJ > 0.15 -- all solutions with Helix = (HelHJ+aveH) + or - 0.03
c   HelHJ < 0.15 -- all solutions with Helix = (HelHJ=minH) + or - 0.03
c
c Find HMin and HMax; and Hlimits
c
	write(Nout,448)
448	 format(/' Now the FOURTH SELECTION RULE is applied to the',
     1    /' Solutions That SATISFY the first THREE rules.'/
     2    ' The HELIX FRACTION should be with in HLIMITS')
	Hmin = 1.0
	Hmax = 0.0
	Hincr = 0.03
	Nsol = Isol
	Do 144 is = 1,Nsol
	   Hel = StrCol(IndSol(is),1)
           If(IBasis.NE.2.AND.IBasis.NE.5)
     1       Hel = StrCol(IndSol(is),1)+StrCol(IndSol(is),2)
	   if(Hel.LE.Hmin) Hmin = Hel
	   if(Hel.GE.Hmax) Hmax = Hel
144     Continue  
c
 	write(Nout,'(/5h Hmin,f7.3,5h Hmax,f7.3,6h HelHJ,f7.3)')
     1          Hmin,Hmax,HelHJ
c
	If (HelHJ.GE.0.65) then
	   Hmax = 1.0
	   Hmin = Hmin - Hincr
	   Write(Nout,'(7h HELIX:,f7.3/)')Hmin-Hincr
	   Hregn = 1
	Endif
	If (HelHJ.GE.0.25.AND.HelHJ.LT.0.65) then
	   Hel = (Hmax + HelHJ) / 2.0
	   HMax = Hel + Hincr
	   HMin = Hel - Hincr
	   Write(Nout,'(7h HELIX:,f7.3/)')Hel
	   Hregn = 2
	Endif
	If (HelHJ.GE.0.15.AND.HelHJ.LT.0.25) then
	   Hel = HelHJ
	   HMax = Hel + Hincr
	   HMin = Hel - Hincr
	   Write(Nout,'(7h HELIX:,f7.3/)')Hel
	   Hregn = 3
	Endif
	If (HelHJ.LT.0.15) then
	   Hel = (Hmin + HelHJ) / 2.0
	   HMax = Hel + Hincr
	   HMin = Hel - Hincr
	   Write(Nout,'(7h HELIX:,f7.3/)')Hel
	   Hregn = 4
	Endif
c	write(Nout,'(/15h HELIX LIMITS: ,f7.3,4h TO ,f7.3/)')Hmin,Hmax
c
c Rule 4  For Solutions in IndSol()
c
 1720   Continue
	IF(abs(Hel-Hmin).GT.0.13) then
	 if(Isol.EQ.0.AND.MinSol.EQ.1) then
	  Write(Nout,1726)Hmin,Hmax
1726	  format(//'  Program CRASHED -- No SOLUTIONS were Obtained',
     1    /' even with HLIMITS of:',f7.3,'-',f7.3,/'  The SOLUTION ',
     2    ' from previous Step should be used')
	  GOTO 999
       	 else
	  If(MinSol.GT.1)MinSol = MinSol - 1
	 endIf 
	Else
	  Write(Nout,1728)Hmin,Hmax
1728	  format(' HLIMITS: ',f7.3,' --->',f7.3/)
	EndIF
	  Isol = 0
          do 152 k=1,Nsec
	     str(k) = 0.0   
152       Continue  
          do 154 k=1,Ncd
	     CalCD(k) = 0.0   
154       Continue  
	  Do 156 is = 1,Nsol
	   Hel = StrCol(IndSol(is),1) 
           If(IBasis.NE.2.AND.IBasis.NE.5)
     1           Hel=StrCol(IndSol(is),1)+StrCol(IndSol(is),2) 
	   If(Hel.GE.Hmin.AND.Hel.LE.HMax) then
	      Isol = Isol + 1
	      SumSS = 0.0
              do 158 k=1,Nsec
		str(k) = str(k) + StrCol(IndSol(is),k)
c	        if(Str(k).GT.0.0) SumSS = SumSS + StrCol(IndSol(is),k)
 	        SumSS = SumSS + StrCol(IndSol(is),k)
158           Continue  
              do 159 k=1,Ncd
		CalCD(k) = CalCD(k) + CDSCol(IndSol(is),k)
159           Continue  
	      Write(Nout,'(3i4,i3,9f7.3)')Isol,is,Ibas(IndSol(is),1),
     1        IBas(IndSol(is),2),(StrCol(IndSol(is),k),k=1,nsec),
     2        SumSS,RmsCol(IndSol(is))
	   EndIf
156       Continue  
c
	  If(Isol.GE.MinSol) then
	     SumSS = 0.0
             do 172 k=1,Nsec
	        str(k) = str(k) / ISol
		SumSS = SumSS + str(k)
172          Continue  
             do 174 k=1,NCD
	       CalCD(k) = CalCD(k) / ISol
174          Continue  
	     RmsCDS = rms(smpl,CalCD,Ncd,MxCD)
	  Else
	     Hmin = Hmin - 0.02
	     Hmax = Hmax + 0.02
	     Hmax = Hmax + 0.02
	     GOTO 1720
	  EndIf
c
 	  Rms_N = rms_nrm(smpl,CalCD,Ncd,MxCD)
c
	  Write(Nout,310)Isol,(StrN(k),k=1,Nsec),SumN,RMSN1
	  Write(Nout,'(1x,A4,9f7.3)')NameP,(Str(k),k=1,nsec),SumSS,RmsCDS
          Write(Nout,450)
c	  Backspace(7)
c	  Write(7,'(1x,A4,9f8.3)')NameP,(Str(k),k=1,nsec),SumSS,
c    1              RmsCDS,Rms_N

310	format(/' Final Solution: Aver. of',i3,' Solns'/
     1	   ' PROT',8(3x,a4))
c
722	format(//'  COMPARISON OF EXPT. AND CALC. CD',
     1   ' SPECTRA (Average of',i3,' Solutions) OF: ',A4//)
c
	If(Iseg.NE.0) then
	   CALL Segment(Str,MxSS,AHel,BStr,HelLn,BetLn)
	   write(Nout,452)AHel,BStr
	   write(Nout,453)
	   write(Nout,456)HelLn,BetLn
	EndIf
c
	write(Nout,450)
	write(Nout,722)Isol,NameP
	CALL Plot(Smpl,CalCD,Wbegin,Wend)
	Write(Nout,'(/31h RMSD between Expt. & Calc. CD=,f7.3)')RmsCDS
	Write(Nout,'(31h NRMSD betwen Expt. & Calc. CD=,f7.3/)')Rms_N
c
c	rewind(2)
c 	write(2,239)
239	format('    WL(nm)   Expt_CD   Calc_CD')
238	format(3f10.2)
	WaveL = Wbegin
	Do ic=1,ncd
	   Backspace(2)
	EndDo
	Do ic=1,ncd
	   write(2,238)WaveL,Smpl(ic),CalCD(ic)
	   WaveL = WaveL - 1
	EndDo
cc
	If(Ibasis.EQ.1) IProt = ' 29 '
	If(Ibasis.EQ.2) IProt = ' 22 '
	If(Ibasis.EQ.3) IProt = ' 37 '
	If(Ibasis.EQ.4) IProt = ' 43 '
	If(Ibasis.EQ.5) IProt = ' 37 '
	If(Ibasis.EQ.6) IProt = ' 42 '
	If(Ibasis.EQ.7) IProt = ' 48 '
	If(Ibasis.EQ.8) IProt = ' CA '
c
	Open(unit=10,file='ProtSS.out',access='APPEND')
        Write(10,440)
440     format(80(1H_)/)
 	Write(10,'(16H SAMPLE:       :,1x,16a4)')Title
 	Write(10,'(26H PROGRAM:      :   SELCON3)')
 	Write(10,'(16H Ref. Prot. Set:,3x,A5)')RefSet
	Write(10,892)(StrN(k),k=1,Nsec)
	Write(10,894)(Str(k),k=1,Nsec)
	Write(*,892)(StrN(k),k=1,Nsec)
	Write(*,894)(Str(k),k=1,Nsec)
	Write(10,'(16h RMSD(Exp-Calc):,f8.3)') RmsCDS
	Write(10,'(16h NRMSD(Exp-Cal):,f8.3)') Rms_N
892	format(' Structure     :',2x,6(2x,a4,2x))
894	format(' Fractions     :',6f8.3)
	Close(10)
C	
c
999	Continue
	END
c***********************************************************************
      SUBROUTINE SVD(M,N,A,W,U,V,IERR,RV1,md1,md2)
      INTEGER I,J,K,L,M,N,II,I1,KK,K1,LL,L1,MN,ITS,IERR
      REAL A(md1,md2),W(md2),U(md1,md2),V(md2,md2),RV1(md2)
      REAL C,F,G,H,S,X,Y,Z,SCALE,ANORM
      LOGICAL MATU,MATV 
C 
C  THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE SVD. 
C  NUM. METH. 14, 403-420 (1970) GOLUB AND REINSCH. 
C  TAKEN DIRECTLY FROM COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS,
C  FORSYTHE,MALCOLM AND MOLER, PRENTICE HALL 1977 
C  SINGULAR VALUE DECOMPOSITION  A= UT S V
C  NM ROW DIMENSION AND AT LEAST EQUAL TO MAX OF N AND M
C  N : NO OF COLUMNS OF A AND U, M: NO OF ROWS
C  A : RECTANGULAR MAT FOR SVD. 
C  MATU AND MATV : SET TO TRUE IF U AND V MATRICES ARE DESIRED
C  W : CONTAINS THE N SINGULAR VALUES OF A (DIAGONAL ELES OF S) 
C      IN CASE OF ERROR EXIT THESE VALUES ARE CORRECT FOR N > IERR
C  IERR : 0 FOR NORMAL RUN , K FOR K TH SINGULAR VALUE NOT DETERMINED 
C 
      IERR=0
      MATU=.TRUE.
      MATV=.TRUE.
      DO 100 I=1,M
	DO 100 J=1,N
100   U(I,J)=A(I,J) 
C  HOUSEHOLDER REDUCTION TO BIDIAGONAL FORM 
      G=0.0 
      SCALE=0.0 
      ANORM=0.0 
C 
      DO 300 I=1,N
	L=I+1 
	RV1(I)=SCALE*G
	G=0.0 
       S=0.0
	SCALE=0.0 
	IF(I.GT.M) GO TO 210
C 
	DO 120 K=I,M
120     SCALE=SCALE+ABS(U(K,I)) 
C 
	IF(SCALE.EQ.0.0) GO TO 210
C 
	DO 130 K=I,M
	  U(K,I)=U(K,I)/SCALE 
130       S =S+U(K,I)*U(K,I)
C 
	  F=U(I,I)
	  G=-SIGN(SQRT(S),F)
	  H=F*G-S 
	  U(I,I)=F-G
	  IF(I.EQ.N) GO TO 190
C 
	  DO 150 J=L,N
	    S=0.0 
	    DO 140 K=I,M
140         S=S+U(K,I)*U(K,J) 
	  F=S/H 
	  DO 150 K=I,M
150         U(K,J)=U(K,J)+F*U(K,I)
C 
190       DO 200 K=I,M
200        U(K,I)=SCALE*U(K,I)
C 
210       W(I)=SCALE*G
	  G=0.0 
	  S=0.0 
	  SCALE=0.0 
	  IF(I.GT.M.OR.I.EQ.N)GO TO 290 
C 
	  DO 220 K=L,N
220         SCALE=SCALE+ABS(U(I,K)) 
C 
	DO 230 K=L,N
	  U(I,K)=U(I,K)/SCALE 
230       S=S+U(I,K)*U(I,K) 
C 
	F=U(I,L)
	G=-SIGN(SQRT(S),F)
	H=F*G-S 
	U(I,L)=F-G
C 
	DO 240 K=L,N
240        RV1(K)=U(I,K)/H
C 
	IF(I.EQ.M) GO TO 270
C 
	DO 260 J=L,M
	  S=0.0 
	  DO 250 K=L,N
250       S=S+U(J,K)*U(I,K) 
	  DO 260 K=L,N
260       U(J,K)=U(J,K)+S*RV1(K)
C 
270     DO 280 K=L,N
280     U(I,K)=SCALE*U(I,K) 
290     ANORM=AMAX1(ANORM,ABS(W(I))+ABS(RV1(I)))
300     CONTINUE
C  ACCUMULATION OF RIGHT HAND TRANSFORMATIONS 
      IF(.NOT.MATV) GO TO 410 
      DO 400 II=1,N 
	I=N+1-II
	IF(I.EQ.N) GO TO 390
	IF (G.EQ.0.0) GO TO 360 
      DO 320 J=L,N
C DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW 
320     V(J,I)=(U(I,J)/U(I,L))/G
      DO 350 J=L,N
	S=0.0 
	DO 340 K=L,N
340     S=S+U(I,K)*V(K,J) 
	DO 350 K=L,N
350     V(K,J)=V(K,J)+S*V(K,I)
360     DO 380 J=L,N
	  V(I,J)=0.0
380       V(J,I)=0.0
390     V(I,I)=1.0
	G=RV1(I)
	L=I 
400     CONTINUE
C  ACCUMULATIONS OF LEFT HAND TRANSFORMATIONS 
410   IF(.NOT.MATU) GOTO 510
      MN=N
      IF(M.LT.N)MN=M
C 
      DO 500 II=1,MN
	I=MN+1-II 
	L=I+1 
	G=W(I)
	IF(I.EQ.N)GO TO 430 
C 
	DO 420 J=L,N
420     U(I,J)=0.0
430     IF(G.EQ.0.0)GO TO 475 
	IF(I.EQ.MN)GO TO 460
	DO 450 J=L,N
	  S=0.0 
	  DO 440 K=L,M
440       S=S+U(K,I)*U(K,J) 
C  DOUBLE DIVISION TI AVOID UNDERFLOW 
	  F=(S/U(I,I))/G
C 
	  DO 450 K=I,M
450       U(K,J)=U(K,J)+F*U(K,I)
460       DO 470 J=I,M
470       U(J,I)=U(J,I)/G 
	  GO TO 490 
475       DO 480 J=I,M
480       U(J,I)=0.0
490       U(I,I)=U(I,I)+1.0 
500       CONTINUE
C  DIAGONALISATION OF THE BIDIAGONAL FORM 
510       DO 700 KK=1,N 
	  K1=N-KK 
	  K=K1+1
	  ITS=0 
C  TEST FOR SPLITTING 
520       DO 530 LL=1,K 
	    L1=K-LL 
	    L=L1+1
	    IF(ABS(RV1(L))+ANORM.EQ.ANORM)GO TO 565 
C  RV1(1) IS ALWAYS ZERO AND SO THERE IS NO EXIT FROM THE BOTTOM OF LOOP
	    IF(ABS(W(L1))+ANORM.EQ.ANORM) GO TO 540 
530         CONTINUE
C  CANCELLATION OF RV1(L)IF L .GT. 1
540      C=0.0
	 S=1.0
	 DO 560 I=L,K 
	   F=S*RV1(I) 
	   RV1(I)=C*RV1(I)
	   IF(ABS(F)+ANORM.EQ.ANORM) GOTO 565 
	   G=W(I) 
	   H=SQRT(F*F+G*G)
	   W(I)=H 
	   C=G/H
	   S=-F/H 
	   IF(.NOT.MATU)GO TO 560 
	   DO 550 J=1,M 
	     Y=U(J,L1)
	     Z=U(J,I) 
	     U(J,L1)=Y*C+Z*S
	     U(J,I)=-Y*S+Z*C
550        CONTINUE 
560      CONTINUE 
C  TEST FOR CONVERGENCE 
565      Z=W(K) 
	 IF(L.EQ.K)GO TO 650
C  SHIFT FROM BOTTOM TWO BY 2 MINOR 
	 IF(ITS.EQ.30) GO TO 1000 
	 ITS=ITS+1
	 X=W(L) 
	 Y=W(K1)
	 G=RV1(K1)
	 H=RV1(K) 
	 F=((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
	 G=SQRT(F*F+1.0)
	 F=((X-Z)*(X+Z)+H*(Y/(F+SIGN(G,F))-H))/X
C  NEXT QR TRANSFORMATION 
	 C=1.0
	 S=1.0
	 DO 600 I1=L,K1 
	   I=I1+1 
	   G=RV1(I) 
	   Y=W(I) 
	   H=S*G
	   G=C*G
	   Z=SQRT(F*F+H*H)
	   RV1(I1)=Z
	   C=F/Z
	   S=H/Z
	   F=X*C+G*S
	  G=-X*S+G*C
	   H=Y*S
	   Y=Y*C
	   IF(.NOT.MATV)GO TO 575 
C 
	   DO 570 J=1,N 
	     X=V(J,I1)
	     Z=V(J,I) 
	     V(J,I1)=X*C+Z*S
	     V(J,I)=-X*S+Z*C
570        CONTINUE 
575        Z=SQRT(F*F+H*H)
	   W(I1)=Z
C  ROTATION CAN BE ARBITRARY IF Z=ZERO
	   IF(Z.EQ.0.0)GO TO 580
	   C=F/Z
	   S=H/Z
580        F=C*G+S*Y
	   X=-S*G+C*Y 
	   IF(.NOT.MATU)GO TO 600 
	   DO 590 J=1,M 
	     Y=U(J,I1)
	     Z=U(J,I) 
	     U(J,I1)=Y*C+Z*S
	     U(J,I)=-Y*S+Z*C
590        CONTINUE 
600        CONTINUE 
	   RV1(L)=0.0 
	   RV1(K)=F 
	   W(K)=X 
	   GO TO 520
C  CONVERGENCE
650        IF(Z.GE.0.0)GO TO 700
C  W(K) IS MADE NONNEGATIVE 
	   W(K)=-Z
	   IF(.NOT.MATV)GO TO 700 
	   DO 690 J=1,N 
690        V(J,K)=-V(J,K) 
700      CONTINUE 
	 GO TO 1001 
C  SET ERROR NOCONVERGENCE TO A 
C  SINGULAR VALUEAFTER 30 ITERATIONS
1000  IERR=K
1001  CONTINUE
      END 
c********************************************************************
c Subroutine to Calculate the Number of Segments
	Subroutine Segment(Str,MxSS,AHel,BStr,HelLn,BetLn)
	dimension Str(MxSS)
	AHel = Str(2) * 100.0 / 4.0
	BStr = Str(4) * 100.0 / 2.0
	If(AHel.LT.0.0) AHel = 0.0
	If(BStr.LT.0.0) BStr = 0.0
	HelLn = 0.0
	If(AHel.NE.0.0)HelLn = (Str(1) + Str(2)) * 100.0 /  AHel
	BetLn = 0.0
	If(BStr.NE.0.0)BetLn = (Str(3) + Str(4)) * 100.0 /  BStr    
	End
c*********************************************************************
c Subroutine to plot CD spectra
	Subroutine Plot(cd1,cd,wbegin,wend)
	Parameter (MxSS=6,MxCD=83,MxPR=61,NCDBeg=260,MxSol=500)
	Parameter (Nin = 3, Nout =4)
	character*1 IPLT
	DIMENSION IPLT(21,60),cd(MxCD),cd1(MxCD),wave(MxCD)
	wavei=wend
	wavef=wbegin
	m=nint(wavef - wavei) + 1
	do 2 i= 1,m      
	  wave(i) = wavei + float(i) - 1
2       continue
	do 5 i=1,21
	 do 5 j=1,60
  	  iplt(i,j)=' '
      	  iplt(i,60)='|'
	  iplt(i,1)='|'
	  iplt(21,j)='-'
	  if (mod(j,6).eq.0) iplt(21,j)='+'
	  iplt(1,j)='-'
	  iplt(11,j)='-'
5	continue
	  iplt(21,1)='+'
	  iplt(11,1)='+'
	  iplt(11,60)='+'
	  iplt(1,1)='+'
	  iplt(1,60)='+'
c Determine Ymax Ymin
	cdmax= ymax(cd,cd1,m,MxCD)
	cdmin= ymin(cd,cd1,m,MxCD)
	facx=  float(wavef-wavei)/60.0
	ymx =amax1(cdmax,abs(cdmin))+0.5
	ymn = -ymx
	facy = ymx / 10.0
c	write(Nout,50)cdmin,cdmax,wavei,wavef,facx,facy,cdrms
	write(Nout,452)
50	format(/' YRANGE:',2f6.2,'; XRANGE:',2f5.0,'; FacX:',f5.3,
     1    '; FacY:',F6.3,/' RmsCD:',f5.3/)

	do 10 i=1,m    
452	format('          LEGEND: **** Calc. CD; oooo Expt. CD;',
     1   ' .... Diff. CD')
c
c Generate CD plot
c
	 x=(wave(i)-wavei)/facx
	 ix=nint(x)+1
	 y=cd(i)/facy
	 iy=nint(y)
	 iy=11-iy 
	 iplt(iy,ix)='*'
c
c Generate CD1 plot - SMPL
c
	 y=cd1(i)/facy
	 iy=nint(y)
	 iy=11-iy
	 iplt(iy,ix)='o'
c
c Generate CD1 - CD  Difference plot
c
	 y=(cd1(i)-CD(i))/facy
	 iy=nint(y)
	 iy=11-iy
	 iplt(iy,ix)='.'
c
10	continue
	write(Nout,250)ymx,(iplt(1,j),j=60,1,-1)
	write(Nout,300)((iplt(i,j),j=60,1,-1),i=2,20)
	write(Nout,250)ymn,(iplt(21,j),j=60,1,-1)
	Wintr = (wavef-wavei )/10.0
	write(Nout,350)wavei,Wintr,wavef
300	format(7x,60a1)
250	format(f7.0,60a1)
350	format(7x,f4.0,'  Wavelength at Intervals of:',f4.1,' nm '
     1        ,14x,f7.0//)
	end
c*********************888
	function ymax(a,a1,n,np)
	dimension a(np),a1(np)
	am=0.0
	do 10 i=1,n
	 if(a1(i).GT.am)am = a1(i)
10	 if(a(i).gt.am)am=a(i)
	ymax=am
	end
c************************
	function ymin(a,a1,n,np)
	dimension a(np),a1(np)
	am=0.0
	do 10 i=1,n
	 if(a1(i).lt.am)am=a1(i)
10	 if(a(i).lt.am)am=a(i)
	ymin=am
	end
c ******************************************************************
c I/O
	Subroutine Message(N)
	If (N.EQ.1) write(*,9023)
9023	 format(80(1h*)
     1 /'         Program SELCON--For analyzing Protein CD spectra'
     1 /'         Ref: Sreerama&Woody, Protein Sci. (1999)' 
     3 /'         Input file: INPUT'
     4 /'         Output files: SELCON.OUT, BASIS, CalcCD.OUT,' 
     5 ,' ProtSS.out')                
	If (N.EQ.2)  then
		Write(*,9038)
 		read(*,*)
	endif
9038	format(80(1h_)//' End of Program'
     1 /6x,' Output in SELCON3.OUT; Final Result in ProtSS.out'
     2/6x,' Run CDSSTR and CONTINLL with this INPUT for Comparison'
     1 /6x,' For additional REFERENCE SETS change IBasis in INPUT'
     1/6x,' Result from different runs are in ProtSS.out'
     1/6x,' MULTIPLE sets of CD data can be analyzed:'
     1 /6x,'   Present multiple CD data sequentially in INPUT'
     2 /6x,' Hit Enter to CONTINUE')
	end
c************************************************************************
	function rms_nrm(s1,s2,n1,np)
c  This function calculates the Normalized rms difference between S1 and S2
c  Wallace et.al. (2003) Prot. Sci., Mao and Wallace (1984)
c  S1 Experiment; S2 Calcuated
c
c NRMSD = Sum((Exp-Cal)**2) / Sum(Exp**2)
c
	dimension s1(np), s2(np)
	rms1=0.0
	Den = 0.0
	do  10 i=1,n1
	   rms1 = rms1 + (s1(i)-s2(i))**2
	   Den  = Den  + s1(i)**2
10      Continue  
crms = sqrt(rms1/float(n1))
	rms_nrm = sqrt(rms1/Den)
	end
